package discRack.presentation.delements;

import discRack.presentation.dpanels.*;

import java.util.*;

/**
 * Represents collection of elements which must be instances of
 * DCollectionElement class.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class DCollection extends DSimpleElement {

   /** The collection of elements. */
   protected ArrayList refCollectionElements=new ArrayList();

   protected transient DPanel controlledPanel;

   protected transient DControlPanel controlPanel;

   public DCollection (String name) {
      super(name);
   }

   /** Adds new element to collection. */
   public void add (DSimpleElement el) {
      refCollectionElements.add(el);
   }

   /** Removes specified element from collection. */
   public void remove (Object el) {
      refCollectionElements.remove(el);
   }

   /** Gets the element that is placed at specified no. from collection. */
   public Object get (int no) {
      try {
         return refCollectionElements.get(no);
      } catch (Exception ex) {
         return null;
      }
   }

   /** Returns the number of elements within collection. */
   public int size () {
      return refCollectionElements.size();
   }

   /** Clears the collection. */
   public void clear () {
      refCollectionElements.clear();
   }

   /**
   * Refreshes the collection.
   *
   * @param elementsToAddOrRemove Set of elements that has to be added to or
   *                              removed from collection.
   * @param append                <tt>true</tt> if adding elements to collection,
   *                              <tt>false</tt> otherwise.
   */
   public void refreshCollection (Set elementsToAddOrRemove,boolean append) {
      if (append) {
         refCollectionElements.addAll(elementsToAddOrRemove);
      } else {
         refCollectionElements.removeAll(elementsToAddOrRemove);
      }
   }

   /**
   * Returns the element specified by ID.
   *
   * @param ID ID of wanted element.
   * @return Wanted element if exist, null otherwise.
   */
   public DCollectionElement getCollectionElement (String ID) {
      DCollectionElement ce=null;
      String ceID;
      Iterator it=refCollectionElements.iterator();
      while (it.hasNext()) {
         DCollectionElement cetmp=(DCollectionElement)it.next();
         ceID=cetmp.getID();
         if (ceID.equals(ID)) {
            ce=cetmp;
            break;
         }
      }
      return ce;
   }

   /**
   * Gets the structure of elements contained within collection,
   * that is the all elements that element is made of.
   */
   public Collection getElementStructure () {
      DSimpleElement el=generateNewElement();
      if (el instanceof DComplexElement) {
         return ((DComplexElement)el).toComplexType();
      }
      else {
         java.util.List l=new ArrayList();
         l.add(el);
         return l;
      }
   }

   /**
   * Sets the collection and all contained elements to be read only or not.
   */
   public void setReadOnly (boolean ro) {
      isReadOnly=ro;
      Iterator it=refCollectionElements.iterator();
      while (it.hasNext()) {
         DSimpleElement el=(DSimpleElement)it.next();
         el.setReadOnly(ro);
      }
   }

   /** Returns the collection of all elements within collection. */
   public Collection toCollection() {
      return refCollectionElements;
   }

   /**
   * Gets elements that can be choosed within table. Default
   * implementation returns all elements within collection.
   */
   public Collection getTableElements() {
      return refCollectionElements;
   }

   /**
   * Generates the new element that made collection. Derived classes
   * has to implement this method to create it's collection element.
   */
   public DSimpleElement generateNewElement() {
      return new DSimpleElement(name);
   }

   /**
   * Some specific things to be done after some action is canceled.
   * Default implementation is nothing to do, and derived classes
   * should implement it's specific actions.
   */
   public void onActionCanceled (DSimpleElement el,int act) {
      return;
   }

   /**
   * Some specific things to be done after element is created.
   * Default implementation is nothing to do, and derived classes
   * should implement it's specific actions.
   */
   public void onElementCreated (DSimpleElement el) {
      return;
   }

   /**
   * Some specific things to be done after element from collection
   * is modified. Default implementation is nothing to do, and derived
   * classes should implement it's specific actions.
   */
   public void onElementModified (DSimpleElement el) {
      return;
   }

   /**
   * Some specific things to be done after element is deleted from
   * collection. Default implementation is nothing to do, and derived
   * classes should implement it's specific actions.
   */
   public void onElementDeleted (DSimpleElement el) {
      return;
   }

   public DPanel getControlledPanel () {
      return controlledPanel;
   }

   public DPanel getControlPanel () {
      return controlPanel;
   }

   /**
   * Returns <tt>true</tt> if there is no elements within collection.
   */
   public boolean isEmpty () {
      return size()==0;
   }

   // First, the controlled panel must be created, and then the control panel
   public DPanel getPanel () {
      controlledPanel=new DTablePanel(this,"",false);
      controlPanel=new DTableControlPanel(this,"",true,false);
      return new DGroupPanel(this,new DPanel[]{
         controlledPanel,controlPanel},toName(),
         false,true);
   }

   public String toString () {
      return name;
   }

}
