/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: InformixConnectionAllocator.java,v 1.2 2003/05/19 16:27:14 wiznogood Exp $
 */

package	com.lutris.appserver.server.sql.informix;

import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.standard.StandardConnectionAllocator;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;

/**
 * An Informix database connection pool.
 *
 * @author	Kyle Clark
 * @since	LBS1.8
 * @version	$Revision: 1.2 $
 */
public class InformixConnectionAllocator extends StandardConnectionAllocator {

    /**
     * The logical database for this connection pool.
     */
    protected InformixLogicalDatabase logicalDatabase;

    /**
     * Creates an Informix database connetion pool.
     *
     * @param logicalDatabase the logical Informix database for which
     * connections are pooled.
     * @param connectionConfig configuration parameters for the
     * database connections.  There are no Informix specific configuration
     * data.
     * @exception ConfigException if bad configuration information is
     * given in the config file.
     */
    protected InformixConnectionAllocator(InformixLogicalDatabase logicalDatabase,
                                          Config connectionConfig)
    throws ConfigException {
        super(logicalDatabase, connectionConfig);
        this.logicalDatabase = logicalDatabase;
    }


    /**
     * Creates a new Informix connection to be added to the connection pool.
     * @exception java.sql.SQLException If a SQL error occures.
     * @see InformixDBConnection
     */
    protected DBConnection createConnection ()
    throws java.sql.SQLException {
        return new InformixDBConnection (this, url, user, password,
                                         maxPreparedStatements,
                                         sqlLogging, generation);
    }

}
