/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: InformixObjectIdAllocator.java,v 1.2 2003/05/19 16:27:14 wiznogood Exp $
 */

package	com.lutris.appserver.server.sql.informix;

import com.lutris.appserver.server.sql.LogicalDatabase;
import com.lutris.appserver.server.sql.standard.StandardObjectIdAllocator;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;

/**
 * Informix Object ids can only be created via this manager.
 * Ensures that all object ids are unique across all
 * objects in this logical database. Also ensures good performance
 * for allocating object ids.
 * <P>
 * The configuration data is specified in the section:
 * <CODE>DatabaseManager.DB.<I>dbName</I>.ObjectId</CODE>
 * <P>
 * Configuration fields are:
 * <UL>
 * <LI> <CODE>CacheSize</CODE> -
 *      The number of object id's to cache between database queries. Optional,
 *	if not specified, then it defaults to 1024.
 * <LI> <CODE>MinValue</CODE> -
 *      The starting number of Object ID allocation. This will only be used
 *	if the Object ID table is empty and thus is useful in development
 *	and testing. Optional, if not specified it defaults to
 *	100000000000000000. Note that the largest number that can be
 *	associated with an OID in LBS is "database: DECIMAL(19,0)"
 * </UL>
 *
 * @author	Kyle Clark
 * @since	LBS1.8
 * @version	$Revision: 1.2 $
 */
public class InformixObjectIdAllocator extends StandardObjectIdAllocator {

    /**
     * Initialize the object id manager.
     *
     * @param objCacheSize The number of object ids to cache between
     *  database queries.
     * @param objIdMinValue The starting number for object ids. Used
     *  only if object id table is empty.
     * @exception ConfigException if bad configuration information is
     *  given in the config file.
     */
    protected InformixObjectIdAllocator (LogicalDatabase logicalDatabase,
                                         Config objIdConfig)
	throws ConfigException {
        super(logicalDatabase, objIdConfig);
    }

}