/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 *
 // compliance with WEBDOCWF begin
 * contains WebDocWf bug fix, transaction needs to be cleared
 * contains WebDocWf fix for parallel delete cascades
 * contains WebDocWf fix for parallel updates during save
 // original line
 //
 // compliance with WEBDOCWF end

 * Contributor(s):
 *
 * $Id: StandardDBTransaction.java,v 1.2 2003/05/19 16:27:14 wiznogood Exp $
 *
 * formatted with JxBeauty (c) johann.langhofer@nextra.at
 */


package  com.lutris.appserver.server.sql.standard;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Vector;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.enhydra.dods.DODS;

import com.lutris.appserver.server.sql.CoreDO;
import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.DBRowUpdateException;
import com.lutris.appserver.server.sql.DBTransaction;
import com.lutris.appserver.server.sql.Transaction;
import com.lutris.dods.builder.generator.query.DataObjectException;


/**
 * Standard implementation of SQL database transaction.
 *
 * @see		DBTransaction
 * @author	Kyle Clark
 * @since	LBS1.8
 * @version	$Revision: 1.2 $
 */
public class StandardDBTransaction  implements DBTransaction {

    // Identifier for this transaction object.
    private int id;

    // Next available identifier.
    private static int nextId;

    // Connection used by this transaction object.
    private DBConnection conn;

    // Is this object still active, or has it been released.
    private boolean released = false;

    // compliance with WEBDOCWF begin
    private Vector vecSortedTransaction = new Vector();

    private HashMap hmpObjectTransaction = new HashMap();

    // original line
    //
    // compliance with WEBDOCWF end
    // Objects that make up this transaction
    private Transaction[] trans =  {};

    // Action (INSERT, UPDATE, DELETE) to apply to each object
    // in the objects array.
    private int[] transAction =  {};

    // Index of next action to apply
    private int transIdx = 0;

    // Amount by which to increment the array size if it is full.
    private final int SIZE_DELTA = 10;

    /*
     * The log channel.
     */
    // v. strahinja, 27 sep 2002      private LogChannel log;
    private Logger logger;

    private static final int INSERT = 1;
    private static final int UPDATE = 2;
    private static final int DELETE = 3;

    /**
     * Construct a transaction object for use on the supplied dB connection.
     *
     * @param conn
     *   The database connection to use.
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     */
    protected StandardDBTransaction (DBConnection conn) throws SQLException  {
        try {
            id = nextId++;
            logDebug("new instance");
            this.conn = conn;
            this.conn.setAutoCommit(false);
        } catch (SQLException sqlExcept) {
            this.conn.handleException(sqlExcept);
            this.conn.release();
            throw  sqlExcept;
        }
    }

//    /**
//     * Public constructor.  Establishes a connection with the default
//     * logical database.
//     * The connection manager must have been initialized with
//     * a default logical database name.
//     *
//     * @exception java.sql.SQLException
//     *   If a database access error occurs.
//     * @exception DatabaseManagerException
//     *   If the default logical database wasn't initialized in the
//     *   Database Manager.
//     * @see DatabaseManager#setDefaultDatabase
//     */
//    public StandardDBTransaction()
//	throws SQLException, DatabaseManagerException {
//
//        this(DODS.getDatabaseManager().allocateConnection());
//    }
//
//
//    /**
//     * Public constructor.  Establishes a connection with the specified
//     * logical database name.
//     *
//     * @param dbName
//     *   Logical database name to access.
//     * @exception java.sql.SQLException
//     *   If a database access error occurs.
//     * @exception DatabaseManagerException
//     *   If a nonexistent logical database name is supplied.
//     */
//    public StandardDBTransaction(String dbName)
//	throws SQLException, DatabaseManagerException {
//
//        this(DODS.getDatabaseManager().allocateConnection(dbName));
//    }
// compliance with WEBDOCWF begin

    /* Method find a DO in the transaction
     *
     * @param transaction
     *   Object that implements transaction interface.
     * @return DO if the oid was in the transaction, null if it was not
     *
     * WebDocWf extension
     */
    public synchronized CoreDO getDO (Transaction transaction) {
        return  getDO(transaction, NONE);
    }
    
    /**
     * Method find a DO in the transaction
     *
     * @param transaction
     *   Object that implements transaction interface.
     * @param action
     *   if not NONE=0, the DO is found only woth the matching action
     * @return DO if the oid was in the transaction, null if it was not
     *
     * WebDocWf extension
     */
    public synchronized CoreDO getDO (Transaction transaction, int action) {
        CoreDO foundDO = null;
        if (transaction instanceof CoreDO) {
            String strOID = ((CoreDO)transaction).getOId().toString();
            if (this.hmpObjectTransaction.containsValue(strOID)) {
                if (action == NONE || ((DOAction)this.hmpObjectTransaction.get(strOID)).getAction()
                    == action) {
                    foundDO = (CoreDO)((DOAction)this.hmpObjectTransaction.get(strOID)).getDO();
                }
            }
        }
        return  foundDO;
    }

    // Parameter for no special action
    private static final int NONE = 0;
    
    // original line
    //
    // compliance with WEBDOCWF end
    /**
     * Method to add an object to the transaction list.
     *
     * @param transaction
     *   Object that implements transaction interface.
     */
    // compliance with WEBDOCWF begin
    private synchronized void add (Transaction transaction, int action) {
        if (transaction instanceof CoreDO) {
            String strOID = ((CoreDO)transaction).getOId().toString();
            if (this.hmpObjectTransaction.containsKey(strOID)) {
                int iDOAction = ((DOAction)this.hmpObjectTransaction.get(strOID)).getAction();
                if (iDOAction == UPDATE && (action == INSERT || action == UPDATE))
                    return;
                if (iDOAction == INSERT && (action == INSERT || action == UPDATE))
                    return;
                if (iDOAction == DELETE)
                    return;
            } else {
                DOAction objDOAction = new DOAction(transaction, action);
                this.hmpObjectTransaction.put(strOID, objDOAction);
                this.vecSortedTransaction.add(objDOAction);
            }
        }
    }

    /**
     * Method to update an object in the database.
     *
     * @param transaction
     *   Object that implements transaction interface.
     */
    public void update (Transaction transaction) {
        add(transaction, UPDATE);
    }

    /**
     * Method to delete an object in the database.
     *
     * @param transaction
     *   Object that implements transaction interface.
     */
    public void delete (Transaction transaction) {
        // If the transaction is an instance of CoreDO and we're
        // doing a DELETE, then check if the oid of this CoreDO
        // is the same as the oid of an existing CoreDO in our
        // transaction. If this is true, do not add this
        // transaction because we are already deleting the DO.
        // This will happen occassionally during complex CASCADE DELETE
        // scenarios when the programmer passes their own
        // DBTransaction to the ***DO.delete() methods.
        add(transaction, DELETE);
    }

    /**
     * Method to insert an object in the database.
     *
     * @param transaction
     *   Object that implements transaction interface.
     */
    public void insert (Transaction transaction) {
        add(transaction, INSERT);
    }

    /**
     * Method to commit upates.
     *
     *
     * contains WebDocWf bug fix, transaction needs to be cleared
     *
     * @exception java.sql.SQLException If a database access error occurs.
     * @exception DBRowUpdateException If a version error occurs.
     */
    public void commit () throws SQLException, DBRowUpdateException {
        try {
            logDebug("commit");
            validate();
            conn.incrRequestCount();                  // Record request
            // compliance with WEBDOCWF begin
            for (int iCounter = 0; iCounter < this.vecSortedTransaction.size(); iCounter++) {
                DOAction objDOAction = (DOAction)this.vecSortedTransaction.get(iCounter);
                switch (objDOAction.getAction()) {
                    case INSERT:
                        objDOAction.getDO().executeInsert(conn);
                        break;
                    case UPDATE:
                        objDOAction.getDO().executeUpdate(conn);
                        break;
                    case DELETE:
                        objDOAction.getDO().executeDelete(conn);
                        break;
                }
            }
            conn.commit();
            // Notify all objects that the transaction succeeded.
            transactionNotify(true);
            
            // Updates cache after transaction's commit
            for (int iCounter = 0; iCounter < this.vecSortedTransaction.size(); iCounter++) {
                DOAction objDOAction = (DOAction)this.vecSortedTransaction.get(iCounter);
                switch (objDOAction.getAction()) {
                    case INSERT:
                        ((CoreDO)objDOAction.getDO()).updateCache();
                        break;
                    case UPDATE:
                        ((CoreDO)objDOAction.getDO()).updateCache();
                        break;
                    case DELETE:
                        ((CoreDO)objDOAction.getDO()).deleteFromCache();
                        break;
                }
            }
        
        } catch (SQLException sqlExcept) {
            handleException(sqlExcept);
            rollback();
            throw  sqlExcept;
        } finally {
            // compliance with WEBDOCWF begin
            this.hmpObjectTransaction = new HashMap();
            this.vecSortedTransaction = new Vector();
        }
    }

    /**
     * Method to rollback changes.
     *
     *
     * contains WebDocWf bug fix, transaction needs to be cleared
     *
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     */
    public void rollback () throws SQLException {
        try {
            logDebug("rollback");
            validate();
            conn.rollback();
            
            // Updates cache after transaction's rollback
            for (int iCounter = 0; iCounter < this.vecSortedTransaction.size(); iCounter++) {
                try {
                    DOAction objDOAction = (DOAction)this.vecSortedTransaction.get(iCounter);
                    ((CoreDO)objDOAction.getDO()).refresh();
                } catch (DataObjectException dataExept){
                    // removes DO from cache if it doesn't exist in the database    
                    if ( dataExept.getMessage().indexOf("DO not found for id") != -1 ) {  // if DO doesn't exist         
                        DOAction objDOAction = (DOAction)this.vecSortedTransaction.get(iCounter);
                        ((CoreDO)objDOAction.getDO()).evict();            
                    } else {
                        throw new SQLException("INTERNAL ERROR: unexpected DataObjectException" );
                    }
                }  
            }
        } catch (SQLException sqlExcept) {
            handleException(sqlExcept);
            throw  sqlExcept;
        } finally {
            // Notify all objects that the transaction failed.
            transactionNotify(false);
            // compliance with WEBDOCWF begin
            this.hmpObjectTransaction = new HashMap();
            this.vecSortedTransaction = new Vector();
            // original line
            //
            // compliance with WEBDOCWF end
        }
    }

    /**
     * Notify all objects in this transaction if the
     * transaction was successfully commited to the database.
     * An object will only be notified once of success or failure
     * even if this method is called multiple times.
     *
     * @param succeeded true if the transaction commited successfully.
     */
    private synchronized void transactionNotify (boolean succeeded) {
        for (int iCounter = 0; iCounter < this.vecSortedTransaction.size(); iCounter++) {
            DOAction objDOAction = (DOAction)this.vecSortedTransaction.get(iCounter);
            switch (objDOAction.getAction()) {
                case INSERT:
                    objDOAction.getDO().finalizeInsert(succeeded);
                    break;
                case UPDATE:
                    objDOAction.getDO().finalizeUpdate(succeeded);
                    break;
                case DELETE:
                    objDOAction.getDO().finalizeDelete(succeeded);
                    break;
            }
        }
    }

    /**
     * Frees all resources consumed by this transaction
     * Connections are returned to the connection pool.
     * Subsequent transactions via this object,
     * will allocate a new set of resources (i.e. connection).
     *
     * contains WebDocWf bug fix, transaction needs to be cleared
     *
     */
    public synchronized void release () {
        try {
            logDebug("release");
            if (!released) {
                conn.reset();
            }
        } catch (SQLException sqlExcept) {
            handleException(sqlExcept);
        } finally {
            conn.release();
            conn = null;
            released = true;
            this.hmpObjectTransaction = new HashMap();
            this.vecSortedTransaction = new Vector();
        }
    }

    /**
     * Exception handeler.  This object is should not be
     * used for subsequent queries if this method returns
     * false.
     *
     * @return boolean True if the exception can be handeled
     *   and the object is still valid, false otherwise.
     */
    public synchronized boolean handleException (SQLException e) {
        logDebug("handle exception");
        return  conn.handleException(e);
    }

    /**
     * Method to ensure this object is still valid.
     * Once this object has been released it cannot be
     * used any more.
     *
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     */
    private void validate () throws SQLException {
        if (released)
            throw  new SQLException("Cannot access DBTransaction object " + "once it has been released.");
    }

    /**
     * If this object has not been <A HREF=#release>released</A>,
     * this method ensures that garbage collection does so.
     */
    protected void finalize () {
        if (!released) {
            release();
        }
    }

    /**
     * Logging. For debuging only, since it effects all Query objects.
     *
     * @param str
     *   The data to log.
     */
    protected void logDebug (String str) {
        if (DODS.getDatabaseManager().debug) {
            if (logger == null)
                logger = DODS.getLogger();
            logger.log(Level.DEBUG, "DBTransaction[" + id + "]: " + str);
        }
    }
    
    private class DOAction {
        private Transaction transDO;
        private int iAction;
        
        public DOAction (Transaction transDOIn, int iActionIn) {
            this.transDO = transDOIn;
            this.iAction = iActionIn;
        }
        
        public Transaction getDO () {
            return  this.transDO;
        }
        
        public int getAction () {
            return  this.iAction;
        }
    }

    /**
     * Name of used database
     */
    protected String databaseName;
    
    /**
     * Method return name of used database
     *
     * @return name of used database
     */
    public String getDatabaseName () {
        return databaseName;
    }
    
    /**
     * Method set name of used database
     *
     * @param dbName name of used database
     */
    public void setDatabaseName (String dbName) {
        databaseName = dbName;
    }
}