/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 */

/*
 *
 * @author    Tanja Jovanovic
 * @version   2.0  07.01.2003.
 *
 */
package org.enhydra.dods.cache;


/**
 * 
 * This class contains data and mechanisms needed for caching data objects and queries.
 *
 */
public interface CacheAdministration {

    /**
     * Returns maximum cache size.
     * 
     * @return Maximum cache size.
     */
    public int getMaxCacheSize();

    /**
     * Get maximum cache size. If the cache is unbounded (the maximum size 
     * is negative), the current cache size is returned.
     *
     * @param real If this parameter is true method return real maximum cache 
     * size, othervise return appropriate value for statistic.
     * 
     * @return maximum cache size.
     */
    public int getMaxCacheSize(boolean real);

    /**
     * Returns size of currently used cache.
     * 
     * @return Size of currently used cache.
     */
    public int getCacheSize();

    /**
     * Sets maximum cache size.
     * 
     * @param maxSize Maximum cache size.
     */
    public void setMaxCacheSize(int maxSize);
    
    /**
     * Returns maximum size of simple query cache.
     * 
     * @return Maximum size of simple query cache.
     */
    public int getMaxSimpleQueryCacheSize();
    
    /**
     * Returns maximum size of simple query cache. If the cache is unbounded 
     * (the maximum size is negative), the current cache size is returned.
     *
     * @param real If this parameter is true method return real maximum cache 
     * size, othervise return appropriate value for statistic.
     * 
     * @return Maximum size of simple query cache.
     */
    public int getMaxSimpleQueryCacheSize(boolean real);
    
    /**
     * Sets maximum size of simple query cache.
     * 
     * @param maxSize maximum size of simple query cache.
     */
    public void setMaxSimpleQueryCacheSize(int maxSize);
    
    /**
     * Returns size of currently used simple query cache.
     * 
     * @return Size of currently used simple query cache.
     */
    public int getSimpleQueryCacheSize();
    
    /**
     * Returns maximum size of complex query cache.
     * 
     * @return Maximum size of complex query cache.
     */
    public int getMaxComplexQueryCacheSize();
    
    /**
     * Returns maximum size of complex query cache. If the cache is unbounded 
     * (the maximum size is negative), the current cache size is returned.
     *
     * @param real If this parameter is true method return real maximum cache 
     * size, othervise return appropriate value for statistic.
     * 
     * @return Maximum size of complex query cache.
     */
    public int getMaxComplexQueryCacheSize(boolean real);
    
    /**
     * Sets maximum size of complex query cache.
     * 
     * @param maxSize maximum size of complex query cache.
     */
    public void setMaxComplexQueryCacheSize(int maxSize);

    /**
     * Returns size of currently used complex query cache.
     * 
     * @return Size of currently used complex query cache.
     */
    public int getComplexQueryCacheSize();

    /**
     * Sets read-only attribute.
     * 
     * @param readOnly New read-only value.
     */
    public void setCacheReadOnly(boolean readOnly);

    /**
     * Returns read-only attribute.
     * 
     * @return read-only attribute
     */
    public boolean getCacheReadOnly();
    
    /**
     * Sets initialQueryCache attribute.
     * 
     * @param initQ New initialQueryCache.
     */
    public void setInitialQueryCache(String initQ);
    
    /**
     * Returns initialQueryCache.
     * 
     * @return initialQueryCache.
     */
    public String getInitialQueryCache();

    /**
     * Returns  
     * @return  
     */
    public boolean isFull();

    /**
     * Get cache type.
     * 
     * @return Cache type.
     */
    public String getCacheType();

}
