
/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 */

package org.enhydra.dods.statistics;

import java.util.Date;

/**
 *
 * This class provides information about table and its statistics.
 *
 * @version 1.0.0.0 
 * @since   Enhydra5.1
 * @author  Tanja Jovanovic
 * @author  Nenad Vico
 */
public interface Statistics {

    /**
     * Returns table name.
     * @return table name.
     */
    public String getTableName();

    /**
     * Sets table name.
     * @param name table name.
     */
    public void setTableName(String name);

    /**
     * Returns total number of queries performed on the table.
     * @return total number of queries performed on the table.
     */
    public int getQueryNum();

    /**
     * Sets total number of queries performed on the table.
     * @param queryNum total number of queries performed on the table.
     */
    public void setQueryNum(int queryNum);

    /**
     * Increases total number of queries performed on the table for one.
     */
    public void incrementQueryNum();
    
    /**
     * Returns total number of queries by oid performed on the table.
     * @return total number of queries performed on the table.
     */
    public int getQueryByOIdNum();

    /**
     * Sets total number of queries by oid performed on the table.
     * @param queryNum total number of queries performed on the table.
     */
    public void setQueryByOIdNum(int queryByOIdNum);

    /**
     * Increases total number of queries by oid performed on the table for one.
     */
    public void incrementQueryByOIdNum();


    /**
     * Returns number of insert statement performed on table.
     * @return number of insert statement performed on table. 
     */
    public int getInsertNum();

    /**
     * Sets number of insert statement performed on table.
     * @param insertNum number of insert performed statement performed on tables.
     */
    public void setInsertNum(int insertNum);

    /**
     * Increases number of insert statement performed on table.
     */
    public void incrementInsertNum();

    /**
     * Returns number of update statement performed on table.
     * @return number of update statement performed on table. 
     */
    public int getUpdateNum();

    /**
     * Sets number of update statement performed on table.
     * @param updateNum number of update performed statement performed on table.
     */
    public void setUpdateNum(int updateNum);

    /**
     * Increases number of update statement performed on table.
     */
    public void incrementUpdateNum();

    /**
     * Returns number of delete statement performed on table.
     * @return number of delete statement performed on table 
     */
    public int getDeleteNum();

    /**
     * Sets number of delete statement performed on table.
     * @param deleteNum number of delete performed statement performed on table.
     */
    public void setDeleteNum(int deleteNum);

    /**
     * Increases number of delete statement performed on table.
     */
    public void incrementDeleteNum();

    /**
     * Returns number of DML operation performed on table.
     * @return number of DML operation performed on table. 
     */
    public int getDMLNum();

    /**
     * Returns number of lazy loadings performed on table.
     * @return number of lazy loadings performed on table.
     */
    public int getLazyLoadingNum();

    /**
     * Sets number of lazy loadings performed on table.
     * @param number of lazy loadings performed on table.
     */
    public void setLazyLoadingNum(int lazyLoadingNum);

    /**
     * Increases number of lazy loadings performed on table for one.
     */
    public void incrementLazyLoadingNum();

    /**
     * Returns time when statistics started.
     * @return time when statistics started.
     */
    public Date getStartTime();

    /**
     * Sets time when statistics started.
     * @param startTime time when statistics started.
     */
    public void setStartTime(Date startTime);

    /**
     * Returns time when statistics has been stopped.
     * @return time when statistics has been stopped.
     */
    public Date getStopTime();

    /**
     * Sets time when statistics has been stopped.
     * @param startTime time when statistics has been stopped.
     */
    public void setStopTime(Date stopTime);

    /**
     * Sets stop time to current time.
     */
    public void stopTime();
    
    /** 
     * Returns average time for executing non-oid query.
     * 
     * @return average time for executing non-oid query.
     */
    public int getQueryAverageTime();
    
    /** 
     * Updates average time for executing non-oid queries.
     * 
     * @param newTime new query time in miliseconds.
     */
    public void updateQueryAverageTime(int newTime);
    
    /** 
     * Returns average time for executing oid query.
     * 
     * @return average time for executing oid query.
     */
    public int getQueryByOIdAverageTime();
    
    /** 
     * Updates average time for executing OId queries and inctrement number 
     * of them by paramether <code>no</code>.
     * 
     * @param newTime new query time in miliseconds for no queries by OId.
     * @param no number of queries by OId.
     */
    public void updateQueryByOIdAverageTime(int newTime, int no);
	/**
	 * Returns cache type.
	 * @return cache type
	 */
	public String getCacheType();

	/**
	 * Sets cache type.
	 * @param type cache type
	 */
	public void setCacheType(String type);
    
}
