
/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 */

package org.enhydra.dods.statistics;

import java.util.Date;

/**
 *
 * This class provides information about table and its statistics.
 *
 * @version 1.0.0.0 
 * @since   Enhydra5.1
 * @author  Tanja Jovanovic
 * @author  Nenad Vico
 */
public class TableStatistics implements Statistics {

	/**
	 * Name of the table.
	 */
	protected String tableName;

	/**
	 * Total number of queries performed on the table.
	 */
	protected int queryNum;

	/**
	 * Total number of queries by oid performed on the table.
	 */
	protected int queryByOIdNum;

	/**
	 * Number of simple queries performed on the table.
	 */
	protected int simpleQueryNum;

	/**
	 * Number of complex queries performed on the table.
	 */
	protected int complexQueryNum;

	/**
	 * Number of insert statement performed on table.
	 */
	protected int insertNum;

	/**
	 * Number of update statement performed on table.
	 */
	protected int updateNum;

	/**
	 * Number of delete statement performed on table.
	 */
	protected int deleteNum;

	/**
	 * Number of lazy loadings performed on table.
	 */
	protected int lazyLoadingNum;

	/**
	 * Time when statistics started.
	 */
	protected Date startTime;

	/**
	 * Time when statistics has been stopped.
	 */
	protected Date stopTime;

	/**
	 * Average time for executing non-oid queries in milliseconds .
	 */
    protected int averageQueryTime;

	/**
	 * Average time for executing non-oid queries in milliseconds .
	 */
    protected int averageQueryByOIdTime;

	/**
     * Constructor which initialize start time of the statistics.
     */
    
	public TableStatistics(String name) {
		startTime = new Date();
		tableName = name;
	}

	/**
	 * Returns table name.
	 * @return table name.
	 */
	public String getTableName() {
		return tableName;
	}

	/**
	 * Sets table name.
	 * @param name table name.
	 */
	public void setTableName(String name) {
		tableName = name;
	}

	/**
	 * Returns total number of queries performed on the table.
	 * @return total number of queries performed on the table.
	 */
	public int getQueryNum() {
		return queryNum;
	}

	/**
	 * Sets total number of queries performed on the table.
	 * @param queryNum total number of queries performed on the table.
	 */
	public void setQueryNum(int queryNum) {
		this.queryNum = queryNum;
	}

	/**
	 * Increases total number of queries performed on the table for one.
	 */
	public void incrementQueryNum() {
		queryNum++;
	}

	/**
	 * Returns total number of queries by oid performed on the table.
	 * @return total number of queries performed on the table.
	 */
	public int getQueryByOIdNum() {
		return queryByOIdNum;
	}

	/**
	 * Sets total number of queries by oid performed on the table.
	 * @param queryNum total number of queries performed on the table.
	 */
	public void setQueryByOIdNum(int queryByOIdNum) {
		this.queryByOIdNum = queryByOIdNum;
	}

	/**
	 * Increases total number of queries by oid performed on the table for one.
	 */
	public void incrementQueryByOIdNum() {
		queryByOIdNum++;
	}

	/**
	 * Returns number of simple queries performed on the table.
	 * @return number simple of queries performed on the table.
	 */
	public int getSimpleQueryNum() {
		return simpleQueryNum;
	}

	/**
	 * Sets number of simple queries performed on the table.
	 * @param simpleQueryNum number of simple queries performed on the table.
	 */
	public void setSimpleQueryNum(int simpleQueryNum) {
		this.simpleQueryNum = simpleQueryNum;
	}

	/**
	 * Increases number of simple queries performed on the table for one.
	 */
	public void incrementSimpleQueryNum() {
		simpleQueryNum++;
	}

	/**
	 * Returns number of complex queries performed on the table.
	 * @return number complex of queries performed on the table.
	 */
	public int getComplexQueryNum() {
		return complexQueryNum;
	}

	/**
	 * Sets number of complex queries performed on the table.
	 * @param complexQueryNum number of complex queries performed on the table.
	 */
	public void setComplexQueryNum(int complexQueryNum) {
		this.complexQueryNum = complexQueryNum;
	}

	/**
	 * Increases number of complex queries performed on the table for one.
	 */
	public void incrementComplexQueryNum() {
		complexQueryNum++;
	}

	/**
	 * Returns number of insert statement performed on table.
	 * @return number of insert statement performed on table. 
	 */
	public int getInsertNum() {
		return insertNum;
	}

	/**
	 * Sets number of insert statement performed on table.
	 * @param insertNum number of insert performed statement performed on tables.
	 */
	public void setInsertNum(int insertNum) {
		this.insertNum = insertNum;
	}

	/**
	 * Increases number of insert statement performed on table.
	 */
	public void incrementInsertNum() {
		insertNum++;
	}

	/**
	 * Returns number of update statement performed on table.
	 * @return number of update statement performed on table. 
	 */
	public int getUpdateNum() {
		return updateNum;
	}

	/**
	 * Sets number of update statement performed on table.
	 * @param updateNum number of update performed statement performed on table.
	 */
	public void setUpdateNum(int updateNum) {
		this.updateNum = updateNum;
	}

	/**
	 * Increases number of update statement performed on table.
	 */
	public void incrementUpdateNum() {
		updateNum++;
	}

	/**
	 * Returns number of delete statement performed on table.
	 * @return number of delete statement performed on table 
	 */
	public int getDeleteNum() {
		return deleteNum;
	}

	/**
	 * Sets number of delete statement performed on table.
	 * @param deleteNum number of delete performed statement performed on table.
	 */
	public void setDeleteNum(int deleteNum) {
		this.deleteNum = deleteNum;
	}

	/**
	 * Increases number of delete statement performed on table.
	 */
	public void incrementDeleteNum() {
		deleteNum++;
	}

	/**
	 * Returns number of DML operation performed on table.
	 * @return number of DML operation performed on table. 
	 */
	public int getDMLNum() {
		return insertNum + updateNum + deleteNum;
	}

	/**
	 * Returns number of lazy loadings performed on table.
	 * @return number of lazy loadings performed on table.
	 */
	public int getLazyLoadingNum() {
		return lazyLoadingNum;
	}

	/**
	 * Sets number of lazy loadings performed on table.
	 * @param number of lazy loadings performed on table.
	 */
	public void setLazyLoadingNum(int lazyLoadingNum) {
		this.lazyLoadingNum = lazyLoadingNum;
	}

	/**
	 * Increases number of lazy loadings performed on table for one.
	 */
	public void incrementLazyLoadingNum() {
		lazyLoadingNum++;
	}

	/**
	 * Returns time when statistics started.
	 * @return time when statistics started.
	 */
	public Date getStartTime() {
		return startTime;
	}

	/**
	 * Sets time when statistics started.
	 * @param startTime time when statistics started.
	 */
	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	/**
	 * Returns time when statistics has been stopped.
	 * @return time when statistics has been stopped.
	 */
	public Date getStopTime() {
		return stopTime;
	}

	/**
	 * Sets time when statistics has been stopped.
	 * @param startTime time when statistics has been stopped.
	 */
	public void setStopTime(Date stopTime) {
		this.stopTime = stopTime;
	}

	/**
	 * Sets stop time to current time.
	 */
	public void stopTime() {
		stopTime = new Date();
	}
    
    /** 
     * Returns average time for executing non-oid query.
     * 
     * @return average time for executing non-oid query.
     */
    public int getQueryAverageTime() {
        return averageQueryTime;
    }
    
    /** 
     * Updates average time for executing non-oid queries.
     * 
     * @param newTime new query time in miliseconds.
     */
    public void updateQueryAverageTime(int newTime) {
        long sum = (queryNum-1)*averageQueryTime+newTime;
        sum = sum/queryNum;
        averageQueryTime = (new Long(sum)).intValue();
    }

    /** 
     * Returns average time for executing oid query.
     * 
     * @return average time for executing oid query.
     */
    public int getQueryByOIdAverageTime() {
        return averageQueryByOIdTime;
    }
    
    /** 
     * Updates average time for executing OId queries and inctrement number 
     * of them by paramether <code>no</code>.
     * 
     * @param newTime new query time in miliseconds for no queries by OId.
     * @param no number of queries by OId.
     */
    public void updateQueryByOIdAverageTime(int newTime, int no) {
        if (no > 0) {
            long sum = (queryByOIdNum)*averageQueryByOIdTime+newTime;
            queryByOIdNum += no;
            sum = sum/queryByOIdNum;
            averageQueryByOIdTime = (new Long(sum)).intValue();
        }
    }

	/**
	 * Returns cache type.
	 * @return cache type
	 */
	public String getCacheType() {
		return "none";
	}

	/**
	 * Sets cache type.
	 * @param type cache type
	 */
	public void setCacheType(String type) {
	}

	/**
	 * Shows content of this class.  
	 * Can be used for debugging.
	 */
	public void show() {
		System.out.println("-------------------------------------------------");
		System.out.println("Table statistics for: " + tableName);
		System.out.println(" queryNum : " + queryNum);
		System.out.println(" queryByOIdNum : " + queryByOIdNum);
		System.out.println(" simpleQueryNum : " + simpleQueryNum);
		System.out.println(" complexQueryNum : " + complexQueryNum);
		System.out.println(" insertNum : " + insertNum);
		System.out.println(" updateNum : " + updateNum);
		System.out.println(" deleteNum : " + deleteNum);
		System.out.println(" lazyLoadingNum : " + lazyLoadingNum);
		System.out.println(" startTime : " + startTime);
		System.out.println(" stopTime : " + stopTime);
        System.out.println(" averageQueryTime : " + averageQueryTime);
        System.out.println(" averageQueryByOIdTime : " + averageQueryByOIdTime);
 	}
}
