/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * C:/develop/gen/org/webdocwf/util/genericstore/data/EnumValueDefinitionQuery.java
 *-----------------------------------------------------------------------------
 */

package org.webdocwf.util.genericstore.data;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Vector;

import org.enhydra.dods.DODS;
import org.webdocwf.dods.access.AccessEvalException;
import org.webdocwf.dods.access.AccessException;
import org.webdocwf.dods.access.AccessRightException;

import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.DBQuery;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.appserver.server.sql.Query;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.dods.builder.generator.query.NonUniqueQueryException;
import com.lutris.dods.builder.generator.query.QueryBuilder;
import com.lutris.dods.builder.generator.query.QueryException;
//end of WebDocWf extension for DODS row instance based security

/**
 * EnumValueDefinitionQuery is used to query the GENERICENUMVALUEDEFINITION 
 * table in the database. It returns objects of type EnumValueDefinitionDO.
 * <P>
 * General usage:
 * <P>
 *     In DODS:
 *        Create a Data Object named "Dog",
 *        and create an Attribute named "Name",
 *        and set that Attribute to "Can be queried."
 *        DODS will then generate the method DogQuery.setQueryName().
 * <P>
 *     In your Business Layer, prepare the query:
 * <P><PRE>
 *             DogQuery dq = new DogQuery();
 *             dq.setQueryName("Rex")
 *             if ( Rex is a reserved name )
 *                 dq.requireUniqueInstance();
 * </PRE>
 *     Then, get the query results one of two ways:
 * <P>
 *         #1:<PRE>
 *             String names = "";
 *             DogDO[] dogs = dq.getDOArray();
 *             for ( int i = 0; i < dogs.length; i++ ) {
 *                 names += dogs[i].getName() + " ";
 *             }
 * </PRE>
 *      or #2:<PRE>
 *             String names = "";
 *             DogDO dog;
 *             while ( null != ( dog = dq.getNextDO() ) ) {
 *                 names += dog.getName() + " ";
 *             }
 * </PRE>
 * <P>
 *     Note:   If <CODE>requireUniqueInstance()</CODE> was called,
 *             then <CODE>getDOArray()</CODE> or <CODE>getNextDO()</CODE>
 *             will throw an exception if more than one "Rex" was found.
 * <P>
 *     Note:   Results of the query will come from the Data Object cache if:
 *             -  The cache is available.
 *             -  Matches were found in the cache.
 *             -  No other tables (Data Objects of other types) were involved
 *                in the query.
 *                This can happen if you extend the <CODE>DogQuery</CODE> class
 *                and you make calls to the <CODE>QueryBuilder</CODE> object
 *                to add SQL involving other tables.
 *             If any of these conditions is not true,
 *             then any results from the query will come from the database.
 * <P>
 *     To reuse the query object, call:
 * <P><PRE>
 *             dq.reset();
 * </PRE>
 * @author administrator
 * @version $Revision: 1.2 $
 */
public class EnumValueDefinitionQuery implements Query {

    private QueryBuilder builder;

    /**
     * Public constructor.
     */
    public EnumValueDefinitionQuery() {
	builder = new QueryBuilder( "GENERICENUMVALUEDEFINITION", "GENERICENUMVALUEDEFINITION.*" );
	try {
		builder.setDatabaseVendor( "MSQL" );
	} catch (Exception e){
		System.out.println("Database vendor error");
	}
// WebDocWf extension for extended wildcard support
// The following line has been put under comments:
	//builder.setStringMatchDetails( "LIKE", "%" );
// The following lines have been added:
	builder.setStringMatchDetails( "LIKE", "%", 
		"_", "", 
		"", "ESCAPE ''");
	builder.setUserStringWildcard( "*"); 
	builder.setUserStringSingleWildcard( "?"); 
	builder.setUserStringSingleWildcardEscape( ""); 
	builder.setUserStringWildcardEscape( ""); 
	builder.setUserStringAppendWildcard( false ); 
	builder.setUserStringTrim( false ); 
// end of WebDocWf extension for extended wildcard support
	reset();
    }
// WebDocWf extension for DODS row instance based security
// The following lines have been added:
/**
  * Constructor for Query with security
  * 
  * @param usr The user for security check
  * 
  * @exception AccessException
  *   The user is not allowed to create a query
  * 
  * WebDocWf extension
  * 
 */
    public EnumValueDefinitionQuery( org.webdocwf.dods.access.User usr ) throws AccessException {
	assertQueryAccess( usr );
	builder = new QueryBuilder( "GENERICENUMVALUEDEFINITION", "GENERICENUMVALUEDEFINITION.*" );
	try {
		builder.setDatabaseVendor( "MSQL" );
	} catch (Exception e){
		System.out.println("Database vendor error");
	}
	builder.setStringMatchDetails( "LIKE", "%", 
		"_", "", 
		"", "ESCAPE ''");
	builder.setUserStringWildcard( "*"); 
	builder.setUserStringSingleWildcard( "?"); 
	builder.setUserStringSingleWildcardEscape( ""); 
	builder.setUserStringWildcardEscape( ""); 
	builder.setUserStringAppendWildcard( false ); 
	builder.setUserStringTrim( false ); 
	user = usr;
	hitDb = true;
	reset(); }
/**
  * Ensure that the given user is allowed to create a query
  * 
  * @param usr The user for security check
  * 
  * @exception AccessException
  *   The user is not allowed to create a query
  * 
  * WebDocWf extension
  * 
 */
    static public void assertQueryAccess( org.webdocwf.dods.access.User usr )
    throws AccessException {
	if (!hasQueryAccess( usr )) throw new AccessRightException("No access !", usr, "Query", null, "EnumValueDefinitionQuery", null, null, null, null, null);
    }
/**
  * Check whether the given user is allowed to create a query
  * 
  * @param usr The user for security check
  * @return Whether the given user is allowed to create a query
  * 
  * WebDocWf extension
  * 
 */
    static public boolean hasQueryAccess( org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAccess( "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO" );
    }
// end of WebDocWf extension for DODS row instance based security

    private ResultSet		resultSet	= null;
    private boolean 		uniqueInstance	= false;
    private EnumValueDefinitionDO[]	DOs		= null;
    private int			arrayIndex	= -1;
    private boolean		needToRun	= true;
    private Vector		cacheHits	= null;

// WebDocWf extension for DODS row instance security
// The following lines has been added:
    private org.webdocwf.dods.access.User user   = null;
/**
  * Get the user of the query
  * 
  * @return The user of the query
  * 
  * WebDocWf extension
  * 
*/
    public org.webdocwf.dods.access.User getUser() { return user; }
// end of WebDocWf extension for DODS row instance security

    private boolean partialCache = false;
    private boolean hitDb = false;

// WebDocWf extension for unique query results without SQL DISTINCT
// The following lines has been added:
    private boolean unique = false;
/**
  * Set the unique flag of the query
  * 
  * @param newUnique The unique flag for the query
  * 
  * WebDocWf extension
  * 
*/
    public void setUnique(boolean newUnique) { unique = newUnique; }
/**
  * Get the unique flag of the query
  * 
  * @return The unique flag of the query
  * 
  * WebDocWf extension
  * 
*/
    public boolean getUnique() { return unique; }
// end of WebDocWf extension for unique query results without SQL DISTINCT

// WebDocWf extension for skipping the n first rows of the result
// The following lines has been added:
    private int readSkip = 0;
/**
  * Set the readSkip number of the query
  * 
  * @param newreadSkip The number of resukts to skip
  * 
  * WebDocWf extension
  * 
*/
    public void setReadSkip(int newReadSkip) { readSkip = newReadSkip; }
/**
  * Get the readSkip number of the query
  * 
  * @return The number of rows which are skipped
  * 
  * WebDocWf extension
  * 
*/
    public int getReadSkip() { return readSkip; }
// end of WebDocWf extension for skipping the n first rows of the result

// WebDocWf extension for select rowcount limit
// The following lines has been added:
    private int databaseLimit = 0;
/**
  * Set the database limit of the query
  * 
  * @param newLimit The limit for the query
  * 
  * WebDocWf extension
  * 
*/
    public void setDatabaseLimit(int newLimit) { databaseLimit = newLimit; }
/**
  * Get the database limit of the query
  * 
  * @return The database limit of the query
  * 
  * WebDocWf extension
  * 
*/
    public int getDatabaseLimit() { return databaseLimit; }
    private boolean databaseLimitExceeded = false;
/**
  * Get the database limit exceeded flag of the query
  * 
  * @return The database limit exceeded flag of the query
  *   True if there would have been more rows than the limit
  * 
  * WebDocWf extension
  * 
*/
    public boolean getDatabaseLimitExceeded() { return databaseLimitExceeded; }
// end of WebDocWf extension for select rowcount limit

    public void hitDatabase() { hitDb = true; }

// WebDocWf extension for extended wildcard support
// The following rows have been added:
    public void setUserStringWildcard(String newUserStringWildcard) 
	{ builder.setUserStringWildcard( newUserStringWildcard ); }
    public void setUserStringSingleWildcard(String newUserStringSingleWildcard) 
	{ builder.setUserStringSingleWildcard( newUserStringSingleWildcard ); }
    public void setUserStringSingleWildcardEscape(String newUserStringSingleWildcardEscape) 
	{ builder.setUserStringSingleWildcardEscape( newUserStringSingleWildcardEscape ); }
    public void setUserStringWildcardEscape(String newUserStringWildcardEscape) 
	{ builder.setUserStringWildcardEscape( newUserStringWildcardEscape ); }

    public void setUserStringAppendWildcard(boolean userStringAppendWildcard ) 
	{ builder.setUserStringAppendWildcard( userStringAppendWildcard ); }
    public void setUserStringTrim(boolean userStringTrim ) 
	{ builder.setUserStringTrim( userStringTrim ); }

    public String getUserStringWildcard() 
	{ return builder.getUserStringWildcard(); }
    public String getUserStringSingleWildcard() 
	{ return builder.getUserStringSingleWildcard(); }
    public String getUserStringSingleWildcardEscape() 
	{ return builder.getUserStringSingleWildcardEscape(); }
    public String getUserStringWildcardEscape() 
	{ return builder.getUserStringWildcardEscape(); }

    public boolean getUserStringAppendWildcard() 
	{ return builder.getUserStringAppendWildcard(); }
    public boolean getUserStringTrim() 
	{ return builder.getUserStringTrim(); }
// end of WebDocWf extension for extended wildcard support

    /**
     * Perform the query on the database, and prepare the
     * array of returned DO objects.
     *
     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    private void runQuery()
    throws DataObjectException, NonUniqueQueryException {
	needToRun = false;
	arrayIndex = -1;

	DBQuery dbQuery = null;
	try {
	    Vector results;
	    if ( partialCache && 0 == cacheHits.size() )
		hitDb = true;
	    if ( hitDb ) {
// WebDocWf extension
// The following lines have been added:
		     try {
        if (user != null) user.restrictQuery(this);
		     } catch ( AccessEvalException ae ) 
	             { throw new DataObjectException("AccessException in restrictQuery", ae);
		     }
// end of WebDocWf extension
		dbQuery = DODS.getDatabaseManager().createQuery();
// WebDocWf extension for database row limit
// Thefollowing line has been added:
           if ((user == null) && (databaseLimit != 0) && (! unique)) setMaxRows(databaseLimit + 1);
// end of WebDocWf extension for database row limit
		dbQuery.query( this );    // invokes executeQuery
	        results = new Vector();
// WebDocWf extension
// The following two lines have been put under comments:
//	        while ( resultSet.next() ) {
//		    results.addElement( EnumValueDefinitionDO.createExisting ( resultSet ) );
// The following lines have been added:
		int tempReadSkip=0; 
		boolean bHasMoreResults = false;
	 	if (! unique)
		{
	          while ( (bHasMoreResults = resultSet.next()) && (databaseLimit==0 || (results.size()<databaseLimit) ) ) {
		    if ( user != null )
		    {
 		     EnumValueDefinitionDO newDO = EnumValueDefinitionDO.createExisting ( resultSet );
		     try {
		      if (!uniqueInstance)
		      {
		        if ( newDO.hasQueryFindAccess( user ) ) 
		        {
			   if (tempReadSkip >= readSkip) 
			   {
  			    results.addElement( newDO );
			   }
			   else
			   {
			    tempReadSkip = tempReadSkip + 1;
			   }
		        }
		      }
		      else //(uniqueInstance)
		      {
		        if ( newDO.hasDOGetAccess( user ) ) 
		        {
			   if (tempReadSkip >= readSkip) 
			   {
   			    results.addElement( newDO );
			   }
			   else
			   {
			    tempReadSkip = tempReadSkip + 1;
			   }
		        }
		      }
		     } catch ( AccessException ae ) 
	             { throw new DataObjectException("AccessException in runQuery()", ae);
		     }
		    }
		    else //no user
		    {
 		     if (tempReadSkip >= readSkip) 
		     {
 		      EnumValueDefinitionDO newDO = EnumValueDefinitionDO.createExisting ( resultSet );
		      results.addElement( newDO );
		     }
		     else
		     {
			tempReadSkip = tempReadSkip + 1;
		     }
   		    }
	         }
		}
		else
		{
		    HashSet hsResult = new HashSet(databaseLimit * 2);
	            while ( (bHasMoreResults = resultSet.next()) && (databaseLimit==0 || (hsResult.size()<databaseLimit) ) ) {
    		        EnumValueDefinitionDO newDO = EnumValueDefinitionDO.createExisting ( resultSet );
		        if ( user != null ) {
    		            try {
		                if (!uniqueInstance) {
		                    if ( newDO.hasQueryFindAccess( user ) ) {
			                if (tempReadSkip >= readSkip) {
                                            if (! hsResult.contains(newDO.getHandle())) {
                                                results.addElement( newDO );
                                                hsResult.add(newDO.getHandle());
                                            } else {
                                                tempReadSkip = tempReadSkip +1;
                                            }
			                } else {
			                    if (! hsResult.contains(newDO.getHandle())) tempReadSkip = tempReadSkip +1;
			                }
		                    }
		                } else {
		                    if ( newDO.hasDOGetAccess( user ) ) {
			                if (tempReadSkip >= readSkip) {
                                            if (! hsResult.contains(newDO.getHandle())) {
                                                results.addElement( newDO );
                                                hsResult.add(newDO.getHandle());
                                            } else {
                                                tempReadSkip = tempReadSkip +1;
                                            }
			                } else {
                                            if (! hsResult.contains(newDO.getHandle())) tempReadSkip = tempReadSkip +1;
			                }
		                    }
                                }
                            } catch ( AccessException ae ) {
	                        throw new DataObjectException("AccessException in runQuery()", ae);
		            }
                        } else {
		            if (tempReadSkip >= readSkip) {
                            	if (! hsResult.contains(newDO.getHandle())) {
                                    results.addElement( newDO );
                                    hsResult.add(newDO.getHandle());
                                } else {
                                    tempReadSkip = tempReadSkip +1;
                                }
			    } else {
			        if (! hsResult.contains(newDO.getHandle())) tempReadSkip = tempReadSkip +1;
			    }
   		        }
	            }
		}
		if ((results.size()==databaseLimit)&& bHasMoreResults) {
                    resultSet.close();
		    databaseLimitExceeded = true;
	        }
	    } else {
		if (readSkip<cacheHits.size())
		{  
		  results = new Vector(cacheHits.subList(readSkip, cacheHits.size()));	 // executeQuery saw cache hits
		}
		else
		{
		  results = new Vector();
		}
	    }
// end of WebDocWf extension
	    if ( results.size() > 1 && uniqueInstance )
		throw new NonUniqueQueryException(
		    "Too many rows returned from database" );
	    DOs = new EnumValueDefinitionDO [ results.size() ];
	    for ( int i = 0; i < results.size(); i++ ) {
		DOs[i] = ( EnumValueDefinitionDO ) results.elementAt( i );
	    }
	    arrayIndex = 0;
	} catch ( SQLException se ) {
	    if (null == se.getSQLState() ) {
		throw new DataObjectException(
		    "Unknown SQLException", se );
	    }
	    if (	se.getSQLState().startsWith("02") &&
			se.getErrorCode() == 100 ) {
		throw new DataObjectException(
		    "Update or delete DO is out of synch", se );
	    } else if (	se.getSQLState().equals("S1000") &&
			se.getErrorCode() == -268 ) {
		throw new DataObjectException(
		    "Integrity constraint violation", se );
	    } else {
		throw new DataObjectException( "Data Object Error", se );
	    }
	} catch ( ObjectIdException oe ) {
	    throw new DataObjectException( "Object ID Error", oe );
	} catch ( DatabaseManagerException de ) {
	    throw new DataObjectException( "Database connection Error", de );
	} finally {
	    if ( null != dbQuery )
		dbQuery.release();
	}
    }


    /**
     * Limit the number of rows (DOs) returned.
     * NOTE: When setting a limit on rows returned by a query,
     * you usually want to use a call to an addOrderBy method
     * to cause the most interesting rows to be returned first.

     * However, the DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.
     * So, setMaxRows also forces the query to hit the database.

     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    public void setMaxRows( int maxRows )
    throws DataObjectException, NonUniqueQueryException {
	hitDb = true;
	builder.setMaxRows( maxRows );
    }

    /**
     * Return array of DOs constructed from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    public EnumValueDefinitionDO[] getDOArray()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	return DOs;
    }

    /**
     * Return successive DOs from array built from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    public EnumValueDefinitionDO getNextDO()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	if ( null == DOs ) {
	    /* This should never happen.
	     * runQuery() should either throw an exception
	     * or create an array of DOs, possibly of zero length.
	     */
	    return null;
	}
	if ( arrayIndex < DOs.length )
	    return DOs[ arrayIndex++ ];
	return null;
    }

    /**
     * Set the OID to query.
     * WARNING!  This method assumes that table <CODE>GENERICENUMVALUEDEFINITION</CODE>
     * has a column named <CODE>"oid"</CODE>.
     * This method is called from the DO classes to retrieve an object by id.
     *
     * @param oid The object id to query.
     */
    public void setQueryOId(ObjectId oid) {
        // Remove from cacheHits any DOs that do not meet this
        // setQuery requirement.
	if ( null == oid )
	    return;
	requireUniqueInstance();
        for ( int i = 0; i < cacheHits.size(); i++ ) {
            EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
            if ( null == DO ) continue;
            boolean equals = true;
	    ObjectId id = DO.getOId();
	    if ( null == id || ! id.equals( oid ) )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database
        // in case there is no cache, or the query involves other tables.
	builder.addWhere( EnumValueDefinitionDO.PrimaryKey, oid.toBigDecimal(),
				QueryBuilder.EQUAL );
    }

    /**
     * Set the object handle to query.
     * This is a variant of setQueryOId().
     *
     * @param handle The string version of the id to query.
     */
    public void setQueryHandle(String handle)
    throws ObjectIdException {
        ObjectId oid = new ObjectId( handle );
        setQueryOId( oid );
    }

    /**
     * Set "unique instance" assertion bit.
     * The first call to the next() method will throw an exception
     * if more than one object was found.
     */
    public void requireUniqueInstance()
    {
	uniqueInstance = true;
    }

    /**
     * Reset the query parameters.
     */
    public void reset() {
	cacheHits	= new Vector();
	
	Enumeration cacheDOs = EnumValueDefinitionDO.getCacheContents();
	if ( null != cacheDOs )
	    while ( cacheDOs.hasMoreElements() )
		cacheHits.addElement( cacheDOs.nextElement() );
	
	DOs		= null;
	uniqueInstance	= false;
	needToRun	= true;
        builder.reset();
    }

    /**
     * Return the appropriate QueryBuilder flag for selecting
     * exact matches (SQL '=') or inexact matches (SQL 'matches').
     */
    private boolean exactFlag( boolean exact ) {
        return exact ? QueryBuilder.EXACT_MATCH : QueryBuilder.NOT_EXACT;
    }



    //
    // Implementation of Query interface
    //

    /**
     * Method to query objects from the database.
     * The following call in runQuery()
     *	    dbQuery.query( this );
     * causes the dbQuery object to invoke
     *      executeQuery()
     *
     * @param conn Handle to database connection.
     * @exception java.sql.SQLException If a database access error occurs.
     */
    public ResultSet executeQuery(DBConnection conn)
        throws SQLException
    {
	resultSet = builder.executeQuery( conn );
        return resultSet;
    }


    /**
     * WARNING!  This method is disabled.
     * It's implementation is forced by the Query interface.
     * This method is disabled for 2 reasons:
     * 1)  the getDOArray() and getNextDO() methods are better
     *     because they return DOs instead of JDBC objects.
     * 2)  the createExisting() method throws an exception
     *     that we cannot reasonably handle here,
     *     and that we cannot throw from here.
     *
     * @param rs JDBC result set from which the next object
     *   will be instantiated.
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an invalid object id was queried from the database.
     */
    public Object next(ResultSet rs)
    throws SQLException, ObjectIdException {
	// TODO: It would be nice to throw an unchecked exception here
	// (an exception that extends RuntimeException)
	// that would be guaranteed to appear during application testing.
	throw new ObjectIdException(
	    "next() should not be used.  Use getNextDO() instead." );
	//return null;
    }

// WebDocWf extension for extended wildcard support
// The following lines have been added:
    /**
     * Convert a String with user wildcards into a string with DB wildcards
     * 
     * @param userSearchValue The string with user wildcards
     * @return The string with DB wildcards
     * 
     * WebDocWf extension
     * 
     */    
    public String convertUserSearchValue( String userSearchValue )
    {
	return builder.convertUserSearchValue( userSearchValue );
    }
    /**
     * Check whether a string contains DB wildcards
     * 
     * @param dbSearchValue The string with possible DB wildcards
     * @return Whether a string contains DB wildcards
     * 
     * WebDocWf extension
     * 
     */    
    public boolean containsWildcards( String dbSearchValue )
    {
	return builder.containsWildcards( dbSearchValue );
    }
// end of WebDocWf extension for extended wildcard support

// WebDocWf extension for query row counter
// The following lines have been added:
    /**
     * Get the rowcount of the query
     * If possible, do it without reading all rows
     * 
     * @return The row count
     * 
     * WebDocWf extension
     * 
     */    
    public int getCount()
    throws NonUniqueQueryException, DataObjectException, SQLException, DatabaseManagerException
    {
	int rowCount=0;
	if (needToRun && databaseLimit==0 && user==null)
	{
	  rowCount = selectCount();
	}
	else
	{
	  if (needToRun) runQuery();
	  rowCount = DOs.length;
	}
	return rowCount;
    }
    /**
     * Get the rowcount of the query by using count(*) in the DB
     * 
     * @return The row count
     * 
     * WebDocWf extension
     * 
     */    
    public int selectCount()
    throws SQLException, DatabaseManagerException
    {
	int rowCount=0;
	String tempClause = builder.getSelectClause();
	builder.setSelectClause(" count(*) as \"counter\" ");
	DBQuery dbQuery = DODS.getDatabaseManager().createQuery();
	dbQuery.query(this);
	resultSet.next();
	rowCount=resultSet.getInt("counter");
	resultSet.close();
	dbQuery.release();
	builder.close();
	resultSet = null;
	builder.setSelectClause(tempClause);
	return rowCount;
    }
// end of WebDocWf extension for query row counter

    /**
     * Set the ENUMTYPEDEFINITIONOID to query, with a QueryBuilder comparison operator.
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQueryENUMTYPEDEFINITIONOID(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO m = DO.getENUMTYPEDEFINITIONOID();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.ENUMTYPEDEFINITIONOID, x, cmp_op );
    }

    /**
     * Set the ENUMTYPEDEFINITIONOID to query.
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQueryENUMTYPEDEFINITIONOID(org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x,String cmp_op)
     */
    public void setQueryENUMTYPEDEFINITIONOID(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQueryENUMTYPEDEFINITIONOID( x, cmp_op );
    }

    /**
     * Set the ENUMTYPEDEFINITIONOID to query
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryENUMTYPEDEFINITIONOID( 
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x )
    throws DataObjectException, QueryException {
	setQueryENUMTYPEDEFINITIONOID( x, true );
    }

    /**
     * Add ENUMTYPEDEFINITIONOID to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByENUMTYPEDEFINITIONOID(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("ENUMTYPEDEFINITIONOID",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add ENUMTYPEDEFINITIONOID to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderByENUMTYPEDEFINITIONOID() {
        builder.addOrderByColumn("ENUMTYPEDEFINITIONOID","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the ENUMTYPEDEFINITIONOID to query.
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQueryENUMTYPEDEFINITIONOID(org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQueryENUMTYPEDEFINITIONOID(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrENUMTYPEDEFINITIONOIDAccess( x, cmp_op, usr );
	setQueryENUMTYPEDEFINITIONOID( x, cmp_op );
    }
    /**
     * Set the ENUMTYPEDEFINITIONOID to query.
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQueryENUMTYPEDEFINITIONOID(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrENUMTYPEDEFINITIONOIDAccess( x, QueryBuilder.EQUAL, usr );
	setQueryENUMTYPEDEFINITIONOID( x );
    }

    /**
     * Set the ENUMTYPEDEFINITIONOID to query, with a QueryBuilder comparison operator.
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQueryENUMTYPEDEFINITIONOID(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrENUMTYPEDEFINITIONOIDAccess( x, cmp_op, usr );
	setQueryENUMTYPEDEFINITIONOID( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrENUMTYPEDEFINITIONOIDAccess(org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrENUMTYPEDEFINITIONOIDAccess(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrENUMTYPEDEFINITIONOIDAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "ENUMTYPEDEFINITIONOID", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrENUMTYPEDEFINITIONOIDAccess(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrENUMTYPEDEFINITIONOIDAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "ENUMTYPEDEFINITIONOID", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrENUMTYPEDEFINITIONOIDAccess(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrENUMTYPEDEFINITIONOIDAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "ENUMTYPEDEFINITIONOID", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrENUMTYPEDEFINITIONOIDAccess(org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrENUMTYPEDEFINITIONOIDAccess(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:ENUMTYPEDEFINITIONOID", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrENUMTYPEDEFINITIONOIDAccess(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:ENUMTYPEDEFINITIONOID", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The ENUMTYPEDEFINITIONOID of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrENUMTYPEDEFINITIONOIDAccess(
				org.webdocwf.util.genericstore.data.EnumTypeDefinitionDO x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:ENUMTYPEDEFINITIONOID", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
//  end of WebDocWf extension for extended wildcard support




    /**
     * Set the BEGINSTATE to query, with a QueryBuilder comparison operator.
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQueryBEGINSTATE(
				boolean x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    boolean m = DO.getBEGINSTATE();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.BEGINSTATE, x, cmp_op );
    }

    /**
     * Set the BEGINSTATE to query.
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQueryBEGINSTATE(boolean x,String cmp_op)
     */
    public void setQueryBEGINSTATE(
				boolean x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQueryBEGINSTATE( x, cmp_op );
    }

    /**
     * Set the BEGINSTATE to query
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryBEGINSTATE( 
				boolean x )
    throws DataObjectException, QueryException {
	setQueryBEGINSTATE( x, true );
    }

    /**
     * Add BEGINSTATE to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByBEGINSTATE(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("BEGINSTATE",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add BEGINSTATE to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderByBEGINSTATE() {
        builder.addOrderByColumn("BEGINSTATE","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the BEGINSTATE to query.
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQueryBEGINSTATE(boolean x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQueryBEGINSTATE(
				boolean x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrBEGINSTATEAccess( x, cmp_op, usr );
	setQueryBEGINSTATE( x, cmp_op );
    }
    /**
     * Set the BEGINSTATE to query.
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQueryBEGINSTATE(
				boolean x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrBEGINSTATEAccess( x, QueryBuilder.EQUAL, usr );
	setQueryBEGINSTATE( x );
    }

    /**
     * Set the BEGINSTATE to query, with a QueryBuilder comparison operator.
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQueryBEGINSTATE(
				boolean x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrBEGINSTATEAccess( x, cmp_op, usr );
	setQueryBEGINSTATE( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrBEGINSTATEAccess(boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrBEGINSTATEAccess(
				boolean x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrBEGINSTATEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "BEGINSTATE", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrBEGINSTATEAccess(
				boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrBEGINSTATEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "BEGINSTATE", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrBEGINSTATEAccess(
				boolean x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrBEGINSTATEAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "BEGINSTATE", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrBEGINSTATEAccess(boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrBEGINSTATEAccess(
				boolean x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:BEGINSTATE", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrBEGINSTATEAccess(
				boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:BEGINSTATE", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The BEGINSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrBEGINSTATEAccess(
				boolean x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:BEGINSTATE", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
//  end of WebDocWf extension for extended wildcard support




    /**
     * Set the ENDSTATE to query, with a QueryBuilder comparison operator.
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQueryENDSTATE(
				boolean x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    boolean m = DO.getENDSTATE();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.ENDSTATE, x, cmp_op );
    }

    /**
     * Set the ENDSTATE to query.
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQueryENDSTATE(boolean x,String cmp_op)
     */
    public void setQueryENDSTATE(
				boolean x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQueryENDSTATE( x, cmp_op );
    }

    /**
     * Set the ENDSTATE to query
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryENDSTATE( 
				boolean x )
    throws DataObjectException, QueryException {
	setQueryENDSTATE( x, true );
    }

    /**
     * Add ENDSTATE to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByENDSTATE(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("ENDSTATE",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add ENDSTATE to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderByENDSTATE() {
        builder.addOrderByColumn("ENDSTATE","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the ENDSTATE to query.
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQueryENDSTATE(boolean x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQueryENDSTATE(
				boolean x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrENDSTATEAccess( x, cmp_op, usr );
	setQueryENDSTATE( x, cmp_op );
    }
    /**
     * Set the ENDSTATE to query.
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQueryENDSTATE(
				boolean x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrENDSTATEAccess( x, QueryBuilder.EQUAL, usr );
	setQueryENDSTATE( x );
    }

    /**
     * Set the ENDSTATE to query, with a QueryBuilder comparison operator.
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQueryENDSTATE(
				boolean x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrENDSTATEAccess( x, cmp_op, usr );
	setQueryENDSTATE( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrENDSTATEAccess(boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrENDSTATEAccess(
				boolean x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrENDSTATEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "ENDSTATE", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrENDSTATEAccess(
				boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrENDSTATEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "ENDSTATE", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrENDSTATEAccess(
				boolean x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrENDSTATEAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "ENDSTATE", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrENDSTATEAccess(boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrENDSTATEAccess(
				boolean x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:ENDSTATE", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrENDSTATEAccess(
				boolean x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:ENDSTATE", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The ENDSTATE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrENDSTATEAccess(
				boolean x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:ENDSTATE", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
//  end of WebDocWf extension for extended wildcard support




    /**
     * Set the LONGNAME to query, with a QueryBuilder comparison operator.
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQueryLONGNAME(
				String x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    String m = DO.getLONGNAME();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.LONGNAME, x, cmp_op );
    }

    /**
     * Set the LONGNAME to query.
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQueryLONGNAME(String x,String cmp_op)
     */
    public void setQueryLONGNAME(
				String x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQueryLONGNAME( x, cmp_op );
    }

    /**
     * Set the LONGNAME to query
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryLONGNAME( 
				String x )
    throws DataObjectException, QueryException {
	setQueryLONGNAME( x, true );
    }

    /**
     * Add LONGNAME to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByLONGNAME(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("LONGNAME",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add LONGNAME to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderByLONGNAME() {
        builder.addOrderByColumn("LONGNAME","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the LONGNAME to query.
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQueryLONGNAME(String x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQueryLONGNAME(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrLONGNAMEAccess( x, cmp_op, usr );
	setQueryLONGNAME( x, cmp_op );
    }
    /**
     * Set the LONGNAME to query.
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQueryLONGNAME(
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrLONGNAMEAccess( x, QueryBuilder.EQUAL, usr );
	setQueryLONGNAME( x );
    }

    /**
     * Set the LONGNAME to query, with a QueryBuilder comparison operator.
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQueryLONGNAME(
				String x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrLONGNAMEAccess( x, cmp_op, usr );
	setQueryLONGNAME( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrLONGNAMEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrLONGNAMEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrLONGNAMEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "LONGNAME", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrLONGNAMEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrLONGNAMEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "LONGNAME", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrLONGNAMEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrLONGNAMEAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "LONGNAME", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrLONGNAMEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrLONGNAMEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:LONGNAME", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrLONGNAMEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:LONGNAME", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrLONGNAMEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:LONGNAME", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
    
    /**
     * Set the LONGNAME to query with a user wildcard string
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchLONGNAME( 
				String x )
    throws DataObjectException, QueryException {
	String y = convertUserSearchValue( x );
	setDBMatchLONGNAME( y );
    }
    /**
     * Set the LONGNAME to query with a user wildcard string
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchLONGNAME( 
				String x , org.webdocwf.dods.access.User usr)
    throws DataObjectException, QueryException, AccessException {
	String y = convertUserSearchValue( x );
	setDBMatchLONGNAME( y, usr );
    }

    /**
     * Set the LONGNAME to query with a DB wildcard string
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchLONGNAME( 
				String x )
    throws DataObjectException, QueryException {
	if (containsWildcards(x) || builder.getUserStringAppendWildcard())
	{
	  builder.addMatchClause( EnumValueDefinitionDO.LONGNAME, x );
	  hitDb = true;
	}
	else
	  setQueryLONGNAME( x, QueryBuilder.EQUAL );
    }
    /**
     * Set the LONGNAME to query with a DB wildcard string
     *
     * @param x The LONGNAME of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchLONGNAME( 
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException {
	assertQueryAttrLONGNAMEAccess( x, containsWildcards(x) || builder.getUserStringAppendWildcard(), usr );
	setDBMatchLONGNAME( x );
    }
    
//  end of WebDocWf extension for extended wildcard support




    /**
     * Set the SHORTNAME to query, with a QueryBuilder comparison operator.
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQuerySHORTNAME(
				String x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    String m = DO.getSHORTNAME();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.SHORTNAME, x, cmp_op );
    }

    /**
     * Set the SHORTNAME to query.
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQuerySHORTNAME(String x,String cmp_op)
     */
    public void setQuerySHORTNAME(
				String x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQuerySHORTNAME( x, cmp_op );
    }

    /**
     * Set the SHORTNAME to query
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQuerySHORTNAME( 
				String x )
    throws DataObjectException, QueryException {
	setQuerySHORTNAME( x, true );
    }

    /**
     * Add SHORTNAME to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderBySHORTNAME(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("SHORTNAME",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add SHORTNAME to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderBySHORTNAME() {
        builder.addOrderByColumn("SHORTNAME","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the SHORTNAME to query.
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQuerySHORTNAME(String x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQuerySHORTNAME(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrSHORTNAMEAccess( x, cmp_op, usr );
	setQuerySHORTNAME( x, cmp_op );
    }
    /**
     * Set the SHORTNAME to query.
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQuerySHORTNAME(
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrSHORTNAMEAccess( x, QueryBuilder.EQUAL, usr );
	setQuerySHORTNAME( x );
    }

    /**
     * Set the SHORTNAME to query, with a QueryBuilder comparison operator.
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQuerySHORTNAME(
				String x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrSHORTNAMEAccess( x, cmp_op, usr );
	setQuerySHORTNAME( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrSHORTNAMEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrSHORTNAMEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrSHORTNAMEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "SHORTNAME", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrSHORTNAMEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrSHORTNAMEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "SHORTNAME", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrSHORTNAMEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrSHORTNAMEAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "SHORTNAME", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrSHORTNAMEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrSHORTNAMEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:SHORTNAME", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrSHORTNAMEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:SHORTNAME", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrSHORTNAMEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:SHORTNAME", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
    
    /**
     * Set the SHORTNAME to query with a user wildcard string
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchSHORTNAME( 
				String x )
    throws DataObjectException, QueryException {
	String y = convertUserSearchValue( x );
	setDBMatchSHORTNAME( y );
    }
    /**
     * Set the SHORTNAME to query with a user wildcard string
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchSHORTNAME( 
				String x , org.webdocwf.dods.access.User usr)
    throws DataObjectException, QueryException, AccessException {
	String y = convertUserSearchValue( x );
	setDBMatchSHORTNAME( y, usr );
    }

    /**
     * Set the SHORTNAME to query with a DB wildcard string
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchSHORTNAME( 
				String x )
    throws DataObjectException, QueryException {
	if (containsWildcards(x) || builder.getUserStringAppendWildcard())
	{
	  builder.addMatchClause( EnumValueDefinitionDO.SHORTNAME, x );
	  hitDb = true;
	}
	else
	  setQuerySHORTNAME( x, QueryBuilder.EQUAL );
    }
    /**
     * Set the SHORTNAME to query with a DB wildcard string
     *
     * @param x The SHORTNAME of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchSHORTNAME( 
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException {
	assertQueryAttrSHORTNAMEAccess( x, containsWildcards(x) || builder.getUserStringAppendWildcard(), usr );
	setDBMatchSHORTNAME( x );
    }
    
//  end of WebDocWf extension for extended wildcard support




    /**
     * Set the KEYVALUE to query, with a QueryBuilder comparison operator.
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQueryKEYVALUE(
				String x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    String m = DO.getKEYVALUE();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.KEYVALUE, x, cmp_op );
    }

    /**
     * Set the KEYVALUE to query.
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQueryKEYVALUE(String x,String cmp_op)
     */
    public void setQueryKEYVALUE(
				String x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQueryKEYVALUE( x, cmp_op );
    }

    /**
     * Set the KEYVALUE to query
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryKEYVALUE( 
				String x )
    throws DataObjectException, QueryException {
	setQueryKEYVALUE( x, true );
    }

    /**
     * Add KEYVALUE to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByKEYVALUE(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("KEYVALUE",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add KEYVALUE to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderByKEYVALUE() {
        builder.addOrderByColumn("KEYVALUE","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the KEYVALUE to query.
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQueryKEYVALUE(String x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQueryKEYVALUE(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrKEYVALUEAccess( x, cmp_op, usr );
	setQueryKEYVALUE( x, cmp_op );
    }
    /**
     * Set the KEYVALUE to query.
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQueryKEYVALUE(
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrKEYVALUEAccess( x, QueryBuilder.EQUAL, usr );
	setQueryKEYVALUE( x );
    }

    /**
     * Set the KEYVALUE to query, with a QueryBuilder comparison operator.
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQueryKEYVALUE(
				String x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrKEYVALUEAccess( x, cmp_op, usr );
	setQueryKEYVALUE( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrKEYVALUEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrKEYVALUEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrKEYVALUEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "KEYVALUE", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrKEYVALUEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrKEYVALUEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "KEYVALUE", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrKEYVALUEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrKEYVALUEAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "KEYVALUE", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrKEYVALUEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrKEYVALUEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:KEYVALUE", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrKEYVALUEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:KEYVALUE", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrKEYVALUEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:KEYVALUE", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
    
    /**
     * Set the KEYVALUE to query with a user wildcard string
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchKEYVALUE( 
				String x )
    throws DataObjectException, QueryException {
	String y = convertUserSearchValue( x );
	setDBMatchKEYVALUE( y );
    }
    /**
     * Set the KEYVALUE to query with a user wildcard string
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchKEYVALUE( 
				String x , org.webdocwf.dods.access.User usr)
    throws DataObjectException, QueryException, AccessException {
	String y = convertUserSearchValue( x );
	setDBMatchKEYVALUE( y, usr );
    }

    /**
     * Set the KEYVALUE to query with a DB wildcard string
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchKEYVALUE( 
				String x )
    throws DataObjectException, QueryException {
	if (containsWildcards(x) || builder.getUserStringAppendWildcard())
	{
	  builder.addMatchClause( EnumValueDefinitionDO.KEYVALUE, x );
	  hitDb = true;
	}
	else
	  setQueryKEYVALUE( x, QueryBuilder.EQUAL );
    }
    /**
     * Set the KEYVALUE to query with a DB wildcard string
     *
     * @param x The KEYVALUE of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchKEYVALUE( 
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException {
	assertQueryAttrKEYVALUEAccess( x, containsWildcards(x) || builder.getUserStringAppendWildcard(), usr );
	setDBMatchKEYVALUE( x );
    }
    
//  end of WebDocWf extension for extended wildcard support




    /**
     * Set the XMLNAME to query, with a QueryBuilder comparison operator.
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    public void setQueryXMLNAME(
				String x,
				String cmp_op )
    throws DataObjectException, QueryException
    {
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
         hitDb = true;
     } else
     {
// end of WebDocWf extension for extended wildcard support
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    EnumValueDefinitionDO DO = ( EnumValueDefinitionDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean found = false;
	    String m = DO.getXMLNAME();
	    found = QueryBuilder.compare( m, x, cmp_op );
	    if ( ! found )
		cacheHits.removeElementAt( i-- );
	}
// WebDocWf extension for extended wildcard support
// The following lines have been added:
     }
// end of WebDocWf extension for extended wildcard support

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
// WebDocWf patch for correct queries in fully cached objects
// The following line has been put under comments:
//	if ( partialCache || hitDb )
// end of WebDocWf patch for correct queries in fully cached objects
	    builder.addWhere( EnumValueDefinitionDO.XMLNAME, x, cmp_op );
    }

    /**
     * Set the XMLNAME to query.
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Instead use 
     * setQueryXMLNAME(String x,String cmp_op)
     */
    public void setQueryXMLNAME(
				String x, boolean exact)
    throws DataObjectException, QueryException
    {
	// TODO:  DELETE THIS METHOD IN A FUTURE RELEASE
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	setQueryXMLNAME( x, cmp_op );
    }

    /**
     * Set the XMLNAME to query
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryXMLNAME( 
				String x )
    throws DataObjectException, QueryException {
	setQueryXMLNAME( x, true );
    }

    /**
     * Add XMLNAME to the ORDER BY clause.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByXMLNAME(boolean direction_flag) {
	hitDb = true;
        builder.addOrderByColumn("XMLNAME",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add XMLNAME to the ORDER BY clause.  This convenience
     * method assumes ascending order.

     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.

     */
    public void addOrderByXMLNAME() {
        builder.addOrderByColumn("XMLNAME","ASC");
    }

// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * Set the XMLNAME to query.
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * setQueryXMLNAME(String x,String cmp_op, org.webdocwf.dods.access.User usr)
     *
     * WebDocWf extension
     *
     */
    public void setQueryXMLNAME(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	assertQueryAttrXMLNAMEAccess( x, cmp_op, usr );
	setQueryXMLNAME( x, cmp_op );
    }
    /**
     * Set the XMLNAME to query.
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void setQueryXMLNAME(
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrXMLNAMEAccess( x, QueryBuilder.EQUAL, usr );
	setQueryXMLNAME( x );
    }

    /**
     * Set the XMLNAME to query, with a QueryBuilder comparison operator.
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     *
     * WebDocWf extension
     *
     */
    public void setQueryXMLNAME(
				String x,
				String cmp_op, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException
    {
	assertQueryAttrXMLNAMEAccess( x, cmp_op, usr );
	setQueryXMLNAME( x, cmp_op );
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Instead use 
     * assertQueryAttrXMLNAMEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrXMLNAMEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	if (!hasQueryAttrXMLNAMEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "XMLNAME", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrXMLNAMEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrXMLNAMEAccess( x, cmp_op, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "XMLNAME", null, null, null, cmp_op);
    }
    /**
     * Ensure the given user is allowed to query the attribute
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @exception AccessException The user is not allowed to query this attribute
     *
     * WebDocWf extension
     *
     */
    public void assertQueryAttrXMLNAMEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	if (!hasQueryAttrXMLNAMEAccess( x, QueryBuilder.EQUAL, usr )) throw new AccessRightException("No access !", usr, "QueryAttr", null, "EnumValueDefinitionDO", "XMLNAME", null, null, null, QueryBuilder.EQUAL);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param exact to use matches or not
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     * @deprecated Instead use 
     * hasQueryAttrXMLNAMEAccess(String x, String cmp_op, org.webdocwf.dods.access.User usr )
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrXMLNAMEAccess(
				String x, boolean exact, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	String cmp_op = QueryBuilder.EQUAL;
	if ( ! exact )
	    cmp_op = QueryBuilder.CASE_INSENSITIVE_CONTAINS;
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:XMLNAME", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrXMLNAMEAccess(
				String x, String cmp_op, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:XMLNAME", x, cmp_op);
    }
    /**
     * Check whether the given user is allowed to query the attribute
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query.
     * @param usr The user for security check
     * @return Whether the given user is allowed to query the attribute
     *
     * WebDocWf extension
     *
     */
    public boolean hasQueryAttrXMLNAMEAccess(
				String x, org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
	return usr.hasQueryAttrAccess("org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO:XMLNAME", x, QueryBuilder.EQUAL);
    }
// end of WebDocWf extension for DODS row instance security

//  WebDocWf extension for extended wildcard support
//  The following lines have been added:
    
    /**
     * Set the XMLNAME to query with a user wildcard string
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchXMLNAME( 
				String x )
    throws DataObjectException, QueryException {
	String y = convertUserSearchValue( x );
	setDBMatchXMLNAME( y );
    }
    /**
     * Set the XMLNAME to query with a user wildcard string
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query with user wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setUserMatchXMLNAME( 
				String x , org.webdocwf.dods.access.User usr)
    throws DataObjectException, QueryException, AccessException {
	String y = convertUserSearchValue( x );
	setDBMatchXMLNAME( y, usr );
    }

    /**
     * Set the XMLNAME to query with a DB wildcard string
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @exception DataObjectException If a database access error occurs.
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchXMLNAME( 
				String x )
    throws DataObjectException, QueryException {
	if (containsWildcards(x) || builder.getUserStringAppendWildcard())
	{
	  builder.addMatchClause( EnumValueDefinitionDO.XMLNAME, x );
	  hitDb = true;
	}
	else
	  setQueryXMLNAME( x, QueryBuilder.EQUAL );
    }
    /**
     * Set the XMLNAME to query with a DB wildcard string
     *
     * @param x The XMLNAME of the GENERICENUMVALUEDEFINITION to query with DB wildcards
     * @param usr The user for security check
     * @exception DataObjectException If a database access error occurs.
     * @exception AccessException The user is not allowed to query this attribute
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    public void setDBMatchXMLNAME( 
				String x, org.webdocwf.dods.access.User usr )
    throws DataObjectException, QueryException, AccessException {
	assertQueryAttrXMLNAMEAccess( x, containsWildcards(x) || builder.getUserStringAppendWildcard(), usr );
	setDBMatchXMLNAME( x );
    }
    
//  end of WebDocWf extension for extended wildcard support



    /**
    * Returns the <code>QueryBuilder</code> that this <code>EnumValueDefinitionQuery</code>
    * uses to construct and execute database queries.
    * <code>EnumValueDefinitionQuery.setQueryXXX</code> methods use 
    * the <code>QueryBuilder</code> to
    * append SQL expressions to the <code>"WHERE"</code> clause to be executed.
    * The <code>QueryBuilder.addEndClause method.</code> can be used to
    * append freeform SQL expressions to the <code>WHERE</code> clause,
    * e.g. "ORDER BY name".
    *
    * <b>Notes regarding cache-enabled DO classes:</b>
    * DO classes can be cache-enabled.  
    * If when using a <code>EnumValueDefinitionQuery</code>, the application developer
    * <b>does not call</b> <code>getQueryBuilder</code>,
    * then <code>EnumValueDefinitionQuery.setQueryXXX</code> methods 
    * simply prune the DO cache and return the remaining results.
    * However, a <code>QueryBuilder</code> builds
    * <CODE>SELECT</CODE> statements for execution by the actual database,
    * and never searches the built-in cache for the DO.
    * So, if the DO class is cache-enabled, and <code>getQueryBuilder</code>
    * is called, this <CODE>EnumValueDefinitionQuery</CODE> object ignores the cache 
    * and hits the actual database.
    */
    public QueryBuilder getQueryBuilder() {
	hitDatabase();
	return builder;
    }

    /**
     * Insert an <CODE>OR</CODE> between <CODE>WHERE</CODE> clauses.
     * Example:  find all the persons named Bob or Robert:
     * <CODE>
     *    PersonQuery pq = new PersonQuery();
     *    pq.setQueryFirstName( "Bob" );
     *    pq.or();
     *    pq.setQueryFirstName( "Robert" );
     * </CODE>
     * 
     * Note:  Calls to <CODE>setQueryXxx</CODE> methods
     * are implicitly <CODE>AND</CODE>ed together,
     * so the following example will always return nothing:
     * <CODE>
     *    PersonQuery pq = new PersonQuery();
     *    pq.setQueryFirstName( "Bob" );
     *    // AND automatically inserted here.
     *    pq.setQueryFirstName( "Robert" );
     * </CODE>

     * 
     * NOTE: The DO cache does not yet support the OR operator.
     * Using the or() method forces the query to hit the database.

     * 
     * @see QueryBuilder to construct more elaborate queries.
     * @author Jay Gunter
     */
    public void or() {
	hitDb = true;
	builder.addWhereOr();
    }

    /**
     * Place an open parenthesis in the <CODE>WHERE</CODE> clause.
     * Example usage:  find all the Bobs and Roberts who are 5 or 50 years old:
     * <CODE>
     *    PersonQuery pq = new PersonQuery();
     *    pq.openParen();
     *       pq.setQueryFirstName( "Bob" );
     *       pq.or();
     *       pq.setQueryFirstName( "Robert" );
     *    pq.closeParen();
     *    // AND automatically inserted here.
     *    pq.openParen();
     *       pq.setQueryAge( 5 );
     *       pq.or();
     *       pq.setQueryAge( 50 );
     *    pq.closeParen();
     * </CODE>

     * 
     * NOTE: The DO cache does not yet support the Open Paren operator.
     * Using the openParen() method forces the query to hit the database.

     * 
     * @see QueryBuilder to construct more elaborate queries.
     * @author Jay Gunter
     */
    public void openParen() {
        hitDb = true;
	builder.addWhereOpenParen();
    }

    /**
     * Place a closing parenthesis in the <CODE>WHERE</CODE> clause.

     * 
     * NOTE: The DO cache does not yet support the Close Paren operator.
     * Using the closeParen() method forces the query to hit the database.

     * 
     * @author Jay Gunter
     */
    public void closeParen() {
        hitDb = true;
	builder.addWhereCloseParen();
    }
}
