/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * contains WebDocWf extensions
 * 
 *-----------------------------------------------------------------------------
 * C:/develop/gen/org/webdocwf/util/genericstore/data/StateTransitionDO.java
 *-----------------------------------------------------------------------------
 */

package org.webdocwf.util.genericstore.data;

import java.math.BigDecimal;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;

import org.enhydra.dods.DODS;
import org.webdocwf.dods.access.AccessEvalException;
import org.webdocwf.dods.access.AccessException;
import org.webdocwf.dods.access.AccessRightException;
import org.webdocwf.util.genericstore.GenericStoreDO;

import com.lutris.appserver.server.sql.CoreDO;
import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.DBRowUpdateException;
import com.lutris.appserver.server.sql.DBTransaction;
import com.lutris.appserver.server.sql.DatabaseManager;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.dods.builder.generator.query.NonUniqueQueryException;
import com.lutris.dods.builder.generator.query.QueryException;
import com.lutris.dods.builder.generator.query.RDBColumn;
import com.lutris.dods.builder.generator.query.RDBColumnValue;
import com.lutris.dods.builder.generator.query.RDBRow;
import com.lutris.dods.builder.generator.query.RDBTable;
import com.lutris.dods.builder.generator.query.RefAssertionException;
// end of WebDocWf extension for DODS row instance security and genric store

/**
 * Data core class, used to set, retrieve the StateTransitionDO information.
 *
 * @version $Revision: 1.2 $
 * @author  administrator
 * @since   org
 */
 public class StateTransitionDO extends org.webdocwf.util.genericstore.GenericStoreDO implements StateTransitionDOI, java.io.Serializable {

    /**
     * static final data members name the table and columns for this DO.
     * By using these members with an instance of the QueryBuilder class,
     * an application can perform straight SQL queries while retaining
     * compile-time checking of table and column usage.
     *
     * Example:  List the Cities containing Persons named Bob:
     *
     *    Using straight SQL with QueryBuilder:
     *      Pro: code runs faster because you create fewer objects
     *      Con: code is less clear
     *
     *         Vector fields = new Vector();
     *         fields.addElement( AddressDO.City );
     *         QueryBuilder qb = new QueryBuilder( fields );
     *         qb.addWhere( PersonDO.FirstName, "Bob" );
     *         qb.addWhere( PersonDO.PrimaryKey, AddressDO.Person );
     *         RDBRow row;
     *         while ( null != ( row = qb.getNextRow() ) ) {
     *             String city = row.get( AddressDO.City ).getString();
     *         }
     *
     *    Using Query/DO classes:
     *      Pro: code is (often) clearer
     *      Con: code runs slower because you create more objects
     *
     *         PersonQuery pq = new PersonQuery();
     *         pq.setQueryFirstName( "Bob" );
     *         PersonDO[] bobs = pq.getDOArray();
     *         for ( int i = 0; i < bobs.length; i++ ) {
     *             AddressQuery aq = new AddressQuery();
     *             aq.setQueryPerson( bobs[i] );
     *             AddressDO addr = aq.getNextDO();
     *             String city = addr.getCity();
     *         }
     */
    static public final RDBTable  table = new RDBTable( "GENERICSTATETRANSITION" );

    /**
     * Return GENERICSTATETRANSITION as the name of the table in the database
     * which contains StateTransitionDO objects.
     * This method overrides CoreDO.getTableName()
     * and is used by CoreDO.executeUpdate() during error handling.
     *
     * @return the database table name
     * @see CoreDO
     * @author Jay Gunter
     */
// WebDocWf enhancement for public tablename
// The following line has been changed
    public String getTableName() {
// before: protected String getTableName() {
	return "GENERICSTATETRANSITION";
    }

   /**
    * static final RDBColumn PrimaryKey for use with QueryBuilder.
    * See example above.
    */
    static public final RDBColumn PrimaryKey = new RDBColumn( table,
					      getPrimaryKeyName() );
    /* RDBColumns for StateTransitionDO attributes are defined below. */

    /* Using a DO (and its Query class) to access a VIEW instead of a TABLE:
     *
     * A DO (and its Query class) can be used to access a VIEW
     * instead of a TABLE.  The Data Object is created as usual in DODS,
     * but the "create table" SQL command for that DO is not used.
     * Instead, you substitute a "create view" command to create a
     * virtual table in the database; this is often done to provide
     * convenient access to a collection of tables joined together.
     *
     * A VIEW usually does not return "oid" and "version" columns;
     * often (but now always) a VIEW is defined to return the "oid" column
     * for one of the tables joined together in the definition of the VIEW.
     * If the isView flag is true, StateTransitionDO.createExisting(ResultSet)
     * will NOT invoke the GenericDO(ResultSet) constructor
     * so to avoid attempting to extract the "oid" and "version" columns
     * from the ResultSet.
     */
    static protected final boolean isView = false;

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private StateTransitionDataStruct data = null;

    /**
     * isReadOnly()
     * Returns true if the data for this object has been marked read-only.
     */
    public boolean isReadOnly() {
	if ( null == data ) return false;
	return data.readOnly;
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected StateTransitionDO ( boolean is_view )
    throws ObjectIdException, DatabaseManagerException {
        super( is_view );
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected StateTransitionDO ()
    throws ObjectIdException, DatabaseManagerException {
        super( isView );
    }

    /**
     * isLoaded()
     * Returns true if the data for this objects has been retrieved
     * from the database.
     */
    public boolean isLoaded() {
	return ( null != data );
    }

    /**
     * loadData()
     * Load the fields for the DO from the database.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public void loadData()
    throws SQLException, ObjectIdException, DataObjectException
    {
	if ( null == data ) {
	    
	    data = new StateTransitionDataStruct ();
	}

	ObjectId id = getOId();
	if ( null == id )
	    return;
	if ( ! isPersistent() )   // DO from createVirgin
	    return;

	// DO from createExisting.  Complain if no record in database.
	StateTransitionQuery query = new StateTransitionQuery ();
	query.setQueryOId( id );
	query.requireUniqueInstance();
	StateTransitionDO obj;
	try {
	    obj = query.getNextDO();
	    if ( null == obj )
		throw new DataObjectException(
		    "StateTransitionDO DO not found for id=" + id );
	    makeIdentical(obj);
	    setVersion(    obj.getVersion() );
	    setNewVersion( obj.getVersion() );

	} catch ( NonUniqueQueryException e ) {
	    throw new ObjectIdException( "Duplicate ObjectId" );
	}
// WebDocWf fix for thread safe lazy loading
// The following lines have been added:

	obj = addToCache( obj );

// end of WebDocWf fix for thread safe lazy loading

    }
    /**
     * Load the actual DO data if necessary.
     * Called by get/set methods.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    protected void checkLoad()
    throws DataObjectException {
	if ( null == data )
	    try {
		loadData();
	    } catch ( Exception e ) {
		throw new DataObjectException(
		    "Unable to load data for StateTransitionDO id=" + getOId() +
		    ", error = " + e.getMessage() );
	    }
    }

    /**
     * Protected constructor used by createExisting(ObjectId) above.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   Should never see this exception since GenericDO.ctor(ObjectId)
     *   never accesses the database.
     */
    protected StateTransitionDO( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	super( id );
    }





    /**
     * A DO class that contains unchanging (static) data from the database
     * will have a cache of DOs representing the entire contents of the table.
     */
    static {
       try {
	    refreshCache();
       } catch ( Exception e ) {
	    // cannot throw from static block
       }
    }

    /**
     * The cache is used automatically by the class.
     * Callers of the create() methods do not know whether
     * the returned DO instance is shared or not.
     * This prevents the existence of multiple instances in memory
     * of the same database object.
     * The cache is instantiated when the first DO instance is created.
     * After the cache is instantiated, the Query class for this DO
     * will search only the cache.
     */
    private static Hashtable cache;

    private static Hashtable oldCache = null;

    /**
     * Queries all rows in table, and for each row
     * creates a DO instance in the cache.
     * For these DOs, data.readOnly = true,
     * which causes set methods to throw an exception.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static void refreshCache()
    throws java.sql.SQLException, DatabaseManagerException, ObjectIdException,
	   DataObjectException {
	    oldCache = cache;
	    cache = new Hashtable();

	    StateTransitionQuery query = new StateTransitionQuery ();
	    query.hitDatabase();
	    StateTransitionDO obj;
	    try {
		while ( null != ( obj = query.getNextDO() ) ) {
		    addToCache( obj );
		    obj.data.readOnly = true;
		}
	    } catch ( NonUniqueQueryException e ) {
		// Since we do not call query.requireUniqueInstance()
		// this should never happen.
		throw new ObjectIdException( "Duplicate ObjectId" );
	    } finally {
	     oldCache = null;
	    }

    }

    /**
     * Add DO to cache.
     * If DO already exists in cache, just the data member is replaced,
     * so that application references to the DO remain valid.
     */
    public static synchronized StateTransitionDO addToCache( StateTransitionDO newDO ) {
	String handle;
        try {
            handle = newDO.getHandle();
        } catch ( DatabaseManagerException e ) {
            return null;
        }
	StateTransitionDO oldDO = null;
	if ( null != oldCache ) {
	    oldDO = ( StateTransitionDO ) oldCache.get( handle );
	} else {
	    oldDO = ( StateTransitionDO ) cache.get( handle );
	}
	if ( null != oldDO ) {
	    oldDO.data = newDO.data;
	    newDO = oldDO;
	}
	cache.put( handle, newDO );
	return newDO;
    }

    /**
     * Retrieve contents of cache as an enumeration of DO objects.
     */
    public static Enumeration getCacheContents() {
	Enumeration ret = null;
	Hashtable h = null;
	h = cache;
	if ( null != h )
	    ret = h.elements();
        return ret;
    }



    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static StateTransitionDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
	return new StateTransitionDO ();
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @param usr The user for security check
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception AccessException
     *   The user is not allowed to create new instances
     *
     * WebDocWf extension
     *
     */
    public static StateTransitionDO createVirgin( org.webdocwf.dods.access.User usr )
    throws DatabaseManagerException, ObjectIdException, AccessException {
        assertDOCreateAccess( usr );
	return createVirgin();
    }
    /**
     * Ensure that the given user is allowed to create new instances
     *
     * @param usr The user for security check
     *
     * @exception AccessException
     *   The user is not allowed to create new instances
     *
     * WebDocWf extension
     *
     */
    public static void assertDOCreateAccess( org.webdocwf.dods.access.User usr )
    throws AccessException {
	if (! hasDOCreateAccess( usr )) throw new AccessRightException("No access !", usr, "Get", null, "org.webdocwf.util.genericstore.data.StateTransitionDO", null, null, null, null, null);
    }
    /**
     * Check whether the given user is allowed to create new instances
     *
     * @param usr The user for security check
     * @return Whether the given user is allowed to create new instances
     *
     * WebDocWf extension
     *
     */
    public static boolean hasDOCreateAccess( org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
      	return usr.hasDOCreateAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO" );
    }
// end of WebDocWf extension for DODS row instance security

// WebDocWf extension for generic store
// The following lines have been addded
    /**
     * Create an instance for an existing row
     * This is a strange hack for having an instance method
     * Maybe there is a better solution ?
     *
     * WebDocWf extension
     *
     */
    public GenericStoreDO createForExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	return createExisting( bd );
    }
// end of WebDocWf extension for generic store

    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a StateTransitionDO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static StateTransitionDO createExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == bd )
	    return null;
	return createExisting( new ObjectId( bd ) );
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a StateTransitionDO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     * @param usr The user for security check
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     * @exception AccessException
     *   The user is not allowed to read the instance existance
     *   
     * WebDocWf extension
     *   
     */
    public static StateTransitionDO createExisting( BigDecimal bd, org.webdocwf.dods.access.User usr )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException, AccessException
    {
	if ( null == bd )
	    return null;
	StateTransitionDO ret = createExisting( new ObjectId( bd ) );
	ret.assertDOGetAccess( usr );
	return ret;
    }
    /**
     * Check whether the given user is allowed to read the DO existance
     * Static function to be called without having the object
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     * @param usr The user for security check
     * @return Whether the given user is allowed to read the DO existance
     *   
     * WebDocWf extension
     *   
     */
    public static boolean hasDOGetAccess( BigDecimal bd, org.webdocwf.dods.access.User usr ) 
    throws AccessEvalException
    {
	StateTransitionDO newDO;
	try {
	  newDO = createExisting( new ObjectId( bd ) );
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDoGetAccess/createExisting !", e, usr, "Get", null, "org.webdocwf.util.genericstore.data.StateTransitionDO", null, null, null, null, null);
	}
	return usr.hasDOGetAccess( newDO );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * The createExisting method is used to create a <CODE>StateTransitionDO</CODE>
     * from a string handle.
     */
    public static StateTransitionDO createExisting( String handle ) {
	StateTransitionDO ret = null;
        try {
            BigDecimal bd = new BigDecimal( handle );
	    ret = createExisting( bd );
        } catch ( Exception e ) {
        }
	return ret;
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * The createExisting method is used to create a <CODE>StateTransitionDO</CODE>
     * from a string handle.
     *   
     * WebDocWf extension
     *   
     */
    public static StateTransitionDO createExisting( String handle, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	StateTransitionDO ret = createExisting( handle );
        if (ret!=null)
        {
          ret.assertDOGetAccess( usr );
        }
	return ret;
    }
    /**
     * Check whether the given user is allowed to read the DO existance
     * Static function to be called without having the object
     *
     * @param handle The string representation of the ObjectId for the object.
     * @param usr The user for security check
     * @return Whether the given user is allowed to read the DO existance
     *   
     * WebDocWf extension
     *   
     */
    public static boolean hasDOGetAccess( String handle, org.webdocwf.dods.access.User usr ) throws AccessEvalException {
	StateTransitionDO newDO;
	try {
	  newDO = createExisting( handle );
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDoGetAccess/createExisting !", e, usr, "Get", null, "org.webdocwf.util.genericstore.data.StateTransitionDO", null, null, null, null, null);
	}
	return usr.hasDOGetAccess( newDO );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * createExisting( ObjectId )
     *
     * Factory method creates a StateTransitionDO object by searching for it
     * in the database using the passed ObjectID value as the primary key.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static StateTransitionDO createExisting( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == id )
	    return null;
	StateTransitionDO ret = null;

	String handle = id.toString();
	ret = findCachedObjectByHandle( handle );
	if ( null != ret )
	    return ret;

	ret = new StateTransitionDO( id );
	ret.setPersistent( true );  // mark DO as persistent (preexisting)
	if ( ! false ) // If not lazy-loading, fetch DO data now.
	    ret.loadData();
// WebDocWf fix for thread safe lazyloading
// The following lines have been put under comments:
//
//	ret = addToCache( ret );
//
// end of WebDocWf fix for thread safe lazyloading
	// unset the GenericDO.dirty flag.
	ret.markClean();
	return ret;
    }

    /**
     * createExisting( ResultSet )
     *
     * Factory method used to create an instance of this class to
     * represent a Data Object already existing in the database.
     *
     * @param rs The ResultSet returned by the Query class for
     * an existing Data Object stored in the database.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static StateTransitionDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == rs )
	    return null;
	StateTransitionDO ret = null;
	if ( isView ) {
	    ret = new StateTransitionDO ();
	    ret.initFromResultSet( rs );
	} else {
	    ret = new StateTransitionDO ( rs );
	}
// WebDocWf fix for thread safe lazyloading
// The following lines have been put under comments:
//
//	ret = addToCache( ret );
//
// end of WebDocWf fix for thread safe lazyloading
	return ret;
    }

    /**
     * createExisting( RDBRow )
     *
     * Factory method creates a StateTransitionDO object by searching for it
     * in the database using the StateTransitionDO.PrimaryKey value
     * in the passed RDBRow.
     *
     * @param RDBRow A row returned by QueryBuilder.getNextRow().
     *
     * @exception DataObjectException
     *   If the RDBRow does not contain a StateTransitionDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static StateTransitionDO createExisting( RDBRow row )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == row )
	    return null;
        RDBColumnValue pk = null;
        try {
	    pk = row.get( StateTransitionDO.PrimaryKey );
	    return createExisting( pk );
        } catch ( Exception e ) {
	    throw new DataObjectException(
		"Cannot create StateTransitionDO, row does not " +
		"contain StateTransitionDO primary key." );
        }
    }

    /**
     * createExisting( RDBColumnValue )
     *
     * Factory method creates a StateTransitionDO object by searching for it
     * in the database using the passed StateTransitionDO.PrimaryKey.
     *
     * @param RDBColumnValue a PrimaryKey column value from a row
     * that was returned by QueryBuilder.getNextRow().
     *
     * @exception DataObjectException
     *   If the RDBColumnValue does not contain a StateTransitionDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static StateTransitionDO createExisting( RDBColumnValue pk )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == pk )
	    return null;
	if ( ! pk.equals( StateTransitionDO.PrimaryKey ) )
	    throw new DataObjectException(
		"Cannot create StateTransitionDO, " +
		"RDBColumnValue is not StateTransitionDO.PrimaryKey." );
	BigDecimal bd = null;
        try {
	    bd = pk.getBigDecimal();
        } catch ( Exception e ) {
	    throw new DataObjectException(
		"Cannot create StateTransitionDO, bad primary key." );
        }
	if ( null == bd )
            return null;
	return createExisting( bd );
    }

    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param data The data struct to copy values from.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static StateTransitionDO createCopy( StateTransitionDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
	StateTransitionDO ret = new StateTransitionDO ();
	ret.data = ( StateTransitionDataStruct ) data.duplicate();
	return ret;
    }

    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static StateTransitionDO createCopy( StateTransitionDO orig )
    throws DatabaseManagerException, ObjectIdException {
	StateTransitionDO ret = new StateTransitionDO ();
	if ( null != orig.data ) {
	    ret.data = ( StateTransitionDataStruct ) orig.data.duplicate();
	}
	return ret;
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     * @param usr The user for security check
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception AccessException
     *   The user is not allowed to make a copy
     *
     * WebDocWf extension
     *
     */
    public static StateTransitionDO createCopy( StateTransitionDO orig, org.webdocwf.dods.access.User usr )
    throws DatabaseManagerException, ObjectIdException, AccessException {
        orig.assertDOCopyAccess ( usr );
	StateTransitionDO ret = createCopy ( orig );
	return ret;
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * reload()
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    public void reload() {
	data = null;
    }


    /**
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle

     *     findCachedObjectByHandle

     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>

     * and <CODE>findCachedObjectByHandle()<CODE>

     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     *
     * @return id of this DO as a string
     *   If an object id can't be allocated for this object.
     */
    public        String  getHandle()
    throws DatabaseManagerException {
	String ret = null;
	    if ( null == getOId() )
		   throw new DatabaseManagerException( "ID not set" );
	    ret = getOId().toString();
        return ret;
    }

   /**
     * hasMatchingHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     */
    public        boolean hasMatchingHandle( String handle ) {
	boolean ret = false;
	    if ( null == getOId() )
		   ret = false;
	    else
		   ret = getOId().toString().equals( handle );
        return ret;
    }


   /**
     * findCachedObjectByHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return <CODE>StateTransitionDO</CODE>
     *   Object if one is found in cache, otherwise null.
     */
    public static StateTransitionDO findCachedObjectByHandle( String handle ){
        if ( null == handle )
            return null;
        if ( null == cache )
            return null;
        return ( StateTransitionDO ) cache.get( handle );
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
   /**
     * findCachedObjectByHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @param usr The user for security check
     * @return <CODE>StateTransitionDO</CODE>
     *   Object if one is found in cache, otherwise null.
     * @exception AccessException
     *   The user is not allowed to read the object existance
     *
     * WebDocWf extension
     *
     */
    public static StateTransitionDO findCachedObjectByHandle( String handle, org.webdocwf.dods.access.User usr )
	throws AccessException {
        StateTransitionDO ret = findCachedObjectByHandle( handle );
	if (ret != null) ret.assertDOGetAccess( usr );
	return ret;
    }
// end of WebDocWf extension for DODS row instance security



    /**
     * makeIdentical()
     *
     * Assigns the DataStruct of an existing DO to this DO.
     * Does not duplicate data. Just assigns the reference.
     *
     * @param orig The original DO.
     *
     */
    protected void makeIdentical( StateTransitionDO orig ) {
	super.makeIdentical(orig);
	data = orig.data;
    }
// WebDocWf extension for writeable fully cached obejcts
    /**
     * makeReadOnly()
     * Mark the object as readonly
     * 
     * WebDocWf extension
     * 
     */
    public void makeReadOnly() {
	if ( null != data ) data.readOnly=true;
    }
    /**
     * Mark the object as readwrite
     * 
     * WebDocWf extension
     * 
     */
    /**
     * makeReadWrite()
     */
    public void makeReadWrite() {
	if ( null != data ) data.readOnly=false;
    }
// end of WebDocWf extension for writeable fully cached obejcts

////////////////////////// data member ENUMVALUEDEFINITIONFROMOID

   
   /* static final RDBColumn ENUMVALUEDEFINITIONFROMOID for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn ENUMVALUEDEFINITIONFROMOID = 
		new RDBColumn( table, "ENUMVALUEDEFINITIONFROMOID", true );

   
   

   /**
    * Get ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @return ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public org.webdocwf.util.genericstore.data.EnumValueDefinitionDO getENUMVALUEDEFINITIONFROMOID () 
   throws DataObjectException {
   
      checkLoad();
   
      return data.ENUMVALUEDEFINITIONFROMOID;
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Get ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @param usr The user for security check
    * @return ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public org.webdocwf.util.genericstore.data.EnumValueDefinitionDO getENUMVALUEDEFINITIONFROMOID ( org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOGetAttrENUMVALUEDEFINITIONFROMOIDAccess( usr );
      org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ret = getENUMVALUEDEFINITIONFROMOID();
      assertDOGetDOValueAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO:ENUMVALUEDEFINITIONFROMOID", ret, usr );
      return ret;
   }
   /**
    * Ensure that the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOGetAttrENUMVALUEDEFINITIONFROMOIDAccess( org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOGetAttrENUMVALUEDEFINITIONFROMOIDAccess( usr )) throw new AccessRightException( "No access", usr, "GetAttr", this, null, "ENUMVALUEDEFINITIONFROMOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
   }

   /**
    * Check whether the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetAttrENUMVALUEDEFINITIONFROMOIDAccess( org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	return hasDOGetAttrAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO:ENUMVALUEDEFINITIONFROMOID", data.ENUMVALUEDEFINITIONFROMOID, usr );
   }
   /**
    * Check whether the given user is allowed to read object in a pointer attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetDOValueENUMVALUEDEFINITIONFROMOIDAccess ( org.webdocwf.dods.access.User usr ) 
   throws AccessEvalException {
	org.webdocwf.util.genericstore.data.EnumValueDefinitionDO val;
	try {
        val = getENUMVALUEDEFINITIONFROMOID();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOGetDOValueENUMVALUEDEFINITIONFROMOIDAccess/getENUMVALUEDEFINITIONFROMOID !", e, usr, "GetAttr", this, null, "ENUMVALUEDEFINITIONFROMOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
	}
      return hasDOGetDOValueAccess(val, usr);
   }
// end of WebDocWf extension for DODS row instance security

   
   /**
    * Set ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @param ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setENUMVALUEDEFINITIONFROMOID ( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONFROMOID )
   throws DataObjectException {
      checkLoad();
      data.ENUMVALUEDEFINITIONFROMOID = (org.webdocwf.util.genericstore.data.EnumValueDefinitionDO) markNewValue(
	data.ENUMVALUEDEFINITIONFROMOID, ENUMVALUEDEFINITIONFROMOID  );
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Set ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    *
    * @param ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    * @param usr The user for security check
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void setENUMVALUEDEFINITIONFROMOID ( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONFROMOID, org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOSetAttrENUMVALUEDEFINITIONFROMOIDAccess( ENUMVALUEDEFINITIONFROMOID, usr );
      setENUMVALUEDEFINITIONFROMOID( ENUMVALUEDEFINITIONFROMOID );
   }
   /**
    * Ensure that the given user is allowed to update the attribute
    *
    * @param ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOSetAttrENUMVALUEDEFINITIONFROMOIDAccess( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONFROMOID, org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOSetAttrENUMVALUEDEFINITIONFROMOIDAccess( ENUMVALUEDEFINITIONFROMOID, usr )) throw new AccessRightException( "No access", usr, "SetAttr", this, null, "ENUMVALUEDEFINITIONFROMOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
   }
   /**
    * Check whether the given user is allowed to update the attribute
    *
    * @param ENUMVALUEDEFINITIONFROMOID of the GENERICSTATETRANSITION
    * @param usr The user for security check
    * @return Whether the given user is allowed to update the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOSetAttrENUMVALUEDEFINITIONFROMOIDAccess( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONFROMOID, org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	try {
	checkLoad();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOSetAttrENUMVALUEDEFINITIONFROMOIDAccess/checkLoad !", e, usr, "SetAttr", this, null, "ENUMVALUEDEFINITIONFROMOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
	}
	return hasDOSetAttrAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO:ENUMVALUEDEFINITIONFROMOID", data.ENUMVALUEDEFINITIONFROMOID,ENUMVALUEDEFINITIONFROMOID,usr );
   }
// end of WebDocWf extension for DODS row instance security
   


////////////////////////// data member ENUMVALUEDEFINITIONTOOID

   
   /* static final RDBColumn ENUMVALUEDEFINITIONTOOID for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn ENUMVALUEDEFINITIONTOOID = 
		new RDBColumn( table, "ENUMVALUEDEFINITIONTOOID", true );

   
   

   /**
    * Get ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @return ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public org.webdocwf.util.genericstore.data.EnumValueDefinitionDO getENUMVALUEDEFINITIONTOOID () 
   throws DataObjectException {
   
      checkLoad();
   
      return data.ENUMVALUEDEFINITIONTOOID;
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Get ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @param usr The user for security check
    * @return ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public org.webdocwf.util.genericstore.data.EnumValueDefinitionDO getENUMVALUEDEFINITIONTOOID ( org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOGetAttrENUMVALUEDEFINITIONTOOIDAccess( usr );
      org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ret = getENUMVALUEDEFINITIONTOOID();
      assertDOGetDOValueAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO:ENUMVALUEDEFINITIONTOOID", ret, usr );
      return ret;
   }
   /**
    * Ensure that the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOGetAttrENUMVALUEDEFINITIONTOOIDAccess( org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOGetAttrENUMVALUEDEFINITIONTOOIDAccess( usr )) throw new AccessRightException( "No access", usr, "GetAttr", this, null, "ENUMVALUEDEFINITIONTOOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
   }

   /**
    * Check whether the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetAttrENUMVALUEDEFINITIONTOOIDAccess( org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	return hasDOGetAttrAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO:ENUMVALUEDEFINITIONTOOID", data.ENUMVALUEDEFINITIONTOOID, usr );
   }
   /**
    * Check whether the given user is allowed to read object in a pointer attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetDOValueENUMVALUEDEFINITIONTOOIDAccess ( org.webdocwf.dods.access.User usr ) 
   throws AccessEvalException {
	org.webdocwf.util.genericstore.data.EnumValueDefinitionDO val;
	try {
        val = getENUMVALUEDEFINITIONTOOID();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOGetDOValueENUMVALUEDEFINITIONTOOIDAccess/getENUMVALUEDEFINITIONTOOID !", e, usr, "GetAttr", this, null, "ENUMVALUEDEFINITIONTOOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
	}
      return hasDOGetDOValueAccess(val, usr);
   }
// end of WebDocWf extension for DODS row instance security

   
   /**
    * Set ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @param ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setENUMVALUEDEFINITIONTOOID ( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONTOOID )
   throws DataObjectException {
      checkLoad();
      data.ENUMVALUEDEFINITIONTOOID = (org.webdocwf.util.genericstore.data.EnumValueDefinitionDO) markNewValue(
	data.ENUMVALUEDEFINITIONTOOID, ENUMVALUEDEFINITIONTOOID  );
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Set ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    *
    * @param ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    * @param usr The user for security check
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void setENUMVALUEDEFINITIONTOOID ( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONTOOID, org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOSetAttrENUMVALUEDEFINITIONTOOIDAccess( ENUMVALUEDEFINITIONTOOID, usr );
      setENUMVALUEDEFINITIONTOOID( ENUMVALUEDEFINITIONTOOID );
   }
   /**
    * Ensure that the given user is allowed to update the attribute
    *
    * @param ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOSetAttrENUMVALUEDEFINITIONTOOIDAccess( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONTOOID, org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOSetAttrENUMVALUEDEFINITIONTOOIDAccess( ENUMVALUEDEFINITIONTOOID, usr )) throw new AccessRightException( "No access", usr, "SetAttr", this, null, "ENUMVALUEDEFINITIONTOOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
   }
   /**
    * Check whether the given user is allowed to update the attribute
    *
    * @param ENUMVALUEDEFINITIONTOOID of the GENERICSTATETRANSITION
    * @param usr The user for security check
    * @return Whether the given user is allowed to update the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOSetAttrENUMVALUEDEFINITIONTOOIDAccess( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONTOOID, org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	try {
	checkLoad();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOSetAttrENUMVALUEDEFINITIONTOOIDAccess/checkLoad !", e, usr, "SetAttr", this, null, "ENUMVALUEDEFINITIONTOOID", null, null, "org.webdocwf.util.genericstore.data.EnumValueDefinitionDO", null);
	}
	return hasDOSetAttrAccess( "org.webdocwf.util.genericstore.data.StateTransitionDO:ENUMVALUEDEFINITIONTOOID", data.ENUMVALUEDEFINITIONTOOID,ENUMVALUEDEFINITIONTOOID,usr );
   }
// end of WebDocWf extension for DODS row instance security
   

    /**
     * Protected constructor.
     *
     * @param rs
     *   Result set from which to obtain product data.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected StateTransitionDO(ResultSet rs)
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
    {
        super(rs);
	initFromResultSet( rs );
//	super.init( rs );
    }

    /**
     * initFromResultSet initializes the data members from GENERICSTATETRANSITION.
     * This code was separated from the ResultSet constructor
     * so that createExisting(ResultSet) could handle VIEWs.
     */
    private void initFromResultSet( ResultSet rs )
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
    {
	// Constructing a DO from a ResultSet means we definitely need the 
	// DataStruct ready for the setXxx methods invoked below.
	data = new StateTransitionDataStruct ();
 
	// writeMemberStuff uses the ResultSetExtraction.template
	// to build up the value for this tag:
	// the value is a series of calls to the DO set methods.

	
	setENUMVALUEDEFINITIONFROMOID( 
	    org.webdocwf.util.genericstore.data.EnumValueDefinitionDO.createExisting( 
		rs.getBigDecimal( 
// WebDocWf bug fix
// The following line has been changed:
			"ENUMVALUEDEFINITIONFROMOID" )
// before: "ENUMVALUEDEFINITIONFROMOID" , 0 )
// end of WebDocWf bug fix
	     )
	);
	
	
	setENUMVALUEDEFINITIONTOOID( 
	    org.webdocwf.util.genericstore.data.EnumValueDefinitionDO.createExisting( 
		rs.getBigDecimal( 
// WebDocWf bug fix
// The following line has been changed:
			"ENUMVALUEDEFINITIONTOOID" )
// before: "ENUMVALUEDEFINITIONTOOID" , 0 )
// end of WebDocWf bug fix
	     )
	);
	

 
        markClean();
	
	
    }        


    

    

    private int[] param = null;

    /**
     * Prepares the statement used to insert this object
     * into the database.
     * @param conn the database connection.
     * @return the insert statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement(DBConnection conn)
            throws SQLException {
	/* 
	 * It would probably be better to have CoreDO implement
	 * 	void addToCache(CoreDO DO) {}
	 * and have each DO that has caching enabled override it as
	 *      void addToCache(CoreDO DO) { cache.put( DO.getOId(), DO ); }
	 * and change CoreDO to invoke addToCache()
	 * when it invokes getInsertStatement().
	 */
      
	

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement( 
	    "insert into GENERICSTATETRANSITION ( ENUMVALUEDEFINITIONFROMOID, ENUMVALUEDEFINITIONTOOID, " + getOIdColumnName() + ", " + getVersionColumnName() + " ) values ( ?, ?, ?, ? )" );

	param = new int[1]; param[0] = 1;
	// writeMemberStuff uses the JDBCsetCalls.template
	// to build up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined in GenericDO.
	try {
	    setPrepStmtParam_DO( stmt, param,
		getENUMVALUEDEFINITIONFROMOID() );
	    setPrepStmtParam_DO( stmt, param,
		getENUMVALUEDEFINITIONTOOID() );


	    /* The ObjecId/Version columns form the primary key.  */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getNewVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     * @param conn the database connection
     * @return the update statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement(DBConnection conn)
            throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(
	    "update GENERICSTATETRANSITION set " + getVersionColumnName() + " = ?, ENUMVALUEDEFINITIONFROMOID = ?, ENUMVALUEDEFINITIONTOOID = ? " +
	    "where " + getOIdColumnName() + " = ? and " + getVersionColumnName() + " = ?" );

	param = new int[1]; param[0] = 1;
	//int[] param = {1};
	// writeMemberStuff builds up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined below.
	try {
	    setPrepStmtParam_int( stmt, param, getNewVersion() );
	    setPrepStmtParam_DO( stmt, param,
		getENUMVALUEDEFINITIONFROMOID() );
	    setPrepStmtParam_DO( stmt, param,
		getENUMVALUEDEFINITIONTOOID() );


	    /* When updating a persistent object, the UPDATE_WHERE_CLAUSE tag
	     * used to build the SQL WHERE clause (above) uses the 
	     * DO's ObjectId and Version to locate the correct row in
	     * the database table.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     * @param conn the database connection
     * @return the delete statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement(DBConnection conn)
            throws SQLException {
        
	cache.remove( getOId().toString() );
	

        String sql =
            "delete from GENERICSTATETRANSITION \n" +
            "where " + getOIdColumnName() + " = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        return stmt;
    }

    

    /*
     * toString - for debugging
     */
/*
    public String toString(){
	String str = "StateTransitionDO:";
	ObjectId oid = getOId();
	String id = "virgin";
	if ( null != oid ) 
	    id = oid.toString();
	str += " OID=" + id;
	if ( null != data ) 
	    str = str + "\n" + indent + "ENUMVALUEDEFINITIONFROMOID=" + ( null == data.ENUMVALUEDEFINITIONFROMOID ? null  : data.ENUMVALUEDEFINITIONFROMOID.toString( indentCount + 1 ) )
+ "\n" + indent + "ENUMVALUEDEFINITIONTOOID=" + ( null == data.ENUMVALUEDEFINITIONTOOID ? null  : data.ENUMVALUEDEFINITIONTOOID.toString( indentCount + 1 ) )
;
        return str + "; " + super.toString();
    }
*/

    /*
     * toString - for debugging
     */
    public String toString(){
        return toString( 1 );
    }
    public String toString( int indentCount ){
        String indent = "";
        for ( int i = 0; i < indentCount; i++ ) {
            indent += ". ";
        }
        String str = indent + "StateTransitionDO:";
        ObjectId oid = getOId();
        String id = "virgin";
        if ( null != oid )
            id = oid.toString();
        str += " OID=" + id;
        if ( null != data )
            str = str + "\n" + indent + "ENUMVALUEDEFINITIONFROMOID=" + ( null == data.ENUMVALUEDEFINITIONFROMOID ? null  : data.ENUMVALUEDEFINITIONFROMOID.toString( indentCount + 1 ) )
+ "\n" + indent + "ENUMVALUEDEFINITIONTOOID=" + ( null == data.ENUMVALUEDEFINITIONTOOID ? null  : data.ENUMVALUEDEFINITIONTOOID.toString( indentCount + 1 ) )
;
        return str + "\n" + indent + "SUPER=" + super.toString( indentCount );
        //return str;
    }

    



    /**
     * A stub method for implementing pre-commit assertions 
     * for this StateTransitionDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for writing to the database.
     */
    protected void okToCommit() 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions 
     * for this StateTransitionDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for deletion from the database.
     */
    protected void okToDelete() 
    throws RefAssertionException { }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * @deprecated Use save() instead.
   */
  public void commit() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line was put under comments
//    modifyDO( null, false );
// The following lines have been inserted:
    DBTransaction dbt = DODS.getDatabaseManager().createTransaction();

    save(dbt);
    dbt.commit();
    dbt.release();
// end of WebDocWf extension for generic store
  }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	save(true);
  }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @param references True if references should be saved with this DO
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save(boolean references) 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line was put under comments
//    modifyDO( null, false, references );
// The following lines have been inserted:
    DBTransaction dbt = DODS.getDatabaseManager().createTransaction();

    save(dbt, references);
    dbt.commit();
    dbt.release();
// end of WebDocWf extension for generic store
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * @deprecated Use save() instead.
   */
  public void commit(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line has been inserted:
    DBTransaction dbtlocal = dbt;
    if (dbtlocal == null) dbtlocal = DODS.getDatabaseManager().createTransaction();
// The following line has been changed:
    modifyDO( dbtlocal, false );
// before: modifyDO( dbt, false );
// The following line has been inserted:
    super.save(dbtlocal);
// end of WebDocWf extension for generic store
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	save(dbt, true);
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @param references True if references of this DO should be saved
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save(DBTransaction dbt, boolean references)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line has been inserted:
    DBTransaction dbtlocal = dbt;
    if (dbtlocal == null) dbtlocal = DODS.getDatabaseManager().createTransaction();
// The following line has been changed:
    modifyDO( dbtlocal, false, references );
// before: modifyDO( dbt, false );
// The following line has been inserted:
    super.save(dbtlocal);
// end of WebDocWf extension for generic store
  }

  /**
   * Deletes the DO from its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {

// WebDocWf extension for generic store
// The following line was put under comments
//    modifyDO( null, true );
// The following lines have been inserted:
    DBTransaction dbt = DODS.getDatabaseManager().createTransaction();
    delete(dbt);
    dbt.commit();
    dbt.release();
// end of WebDocWf extension for generic store
  }

  /**
   * Deletes the DO from its table.
   * The transaction is likely provided by the delete() method of another DO
   * which references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following lines have been inserted:
    DBTransaction dbtlocal = dbt;
    if (dbtlocal == null) dbtlocal = DODS.getDatabaseManager().createTransaction();
    super.delete(dbtlocal);
// The following line has been changed:
    modifyDO( dbtlocal, true );
// before:    modifyDO( dbt, true );
// end of WebDocWf extension for generic store
  }

      /**
     * A stub method for implementing pre-commit assertions 
     * for the ENUMVALUEDEFINITIONFROMOID data member.
     * Implement this stub to throw an RefAssertionException for cases
     * where ENUMVALUEDEFINITIONFROMOID is not valid for writing to the database.
     */
    protected void okToCommitENUMVALUEDEFINITIONFROMOID( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO member ) 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions 
     * for the ENUMVALUEDEFINITIONFROMOID data member.
     * Implement this stub to throw an RefAssertionException for cases
     * where ENUMVALUEDEFINITIONFROMOID is not valid for deletion from the database.
     */
    protected void okToDeleteENUMVALUEDEFINITIONFROMOID( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO member ) 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-commit assertions 
     * for the ENUMVALUEDEFINITIONTOOID data member.
     * Implement this stub to throw an RefAssertionException for cases
     * where ENUMVALUEDEFINITIONTOOID is not valid for writing to the database.
     */
    protected void okToCommitENUMVALUEDEFINITIONTOOID( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO member ) 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions 
     * for the ENUMVALUEDEFINITIONTOOID data member.
     * Implement this stub to throw an RefAssertionException for cases
     * where ENUMVALUEDEFINITIONTOOID is not valid for deletion from the database.
     */
    protected void okToDeleteENUMVALUEDEFINITIONTOOID( org.webdocwf.util.genericstore.data.EnumValueDefinitionDO member ) 
    throws RefAssertionException { }



  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
// WebDocWf extension for save without references
// The following line has been modofied
  protected void modifyDO( DBTransaction dbt, boolean delete)
// before:  protected void modifyDO( DBTransaction dbt, boolean delete )
// end of WebDocWf extension for save without references
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	modifyDO( dbt, delete, true);
  }
  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @param references True if references should be saved
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
// WebDocWf extension for save without references
// The following line has been modofied
  protected void modifyDO( DBTransaction dbt, boolean delete, boolean references )
// before:  protected void modifyDO( DBTransaction dbt, boolean delete )
// end of WebDocWf extension for save without references
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    if ( delete )
	okToDelete();
    else
	okToCommit();
    boolean ownTransaction = false;
    try {
      if ( null == dbt ) {
	DatabaseManager dbm = DODS.getDatabaseManager();
        dbt = dbm.createTransaction();      // create a transaction
	ownTransaction = true;
      } 
// WebDocWf fix for circular references
// The following lines have been inserted
	  else
      {
        CoreDO foundDO = dbt.getDO(this);
        if (foundDO!=null) return;
      }
// end of WebDocWf fix for circular references
      if ( null == dbt )
	throw new DatabaseManagerException(
		"DatabaseManager.createTransaction returned null." );
      if ( delete ) {
	  // Code to perform cascading deletes is generated here
	  // if cascading deletes are not supported by the database.      
	  
	  // The following line keeps the compiler happy 
	  // when the CASCADING_DELETES tag is empty.
          if ( false )
	      throw new QueryException("XXX");
      } else {
// WebDocWf extension for save without references
// The following line has been inserted
	  if (references) {
// end of WebDocWf extension for save without references
	  // commit referenced DOs.
	  	org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONFROMOID_DO = getENUMVALUEDEFINITIONFROMOID();
	if ( null != ENUMVALUEDEFINITIONFROMOID_DO ) {
	    if ( ENUMVALUEDEFINITIONFROMOID_DO.isLoaded() ) {
		okToCommitENUMVALUEDEFINITIONFROMOID( ENUMVALUEDEFINITIONFROMOID_DO );
		ENUMVALUEDEFINITIONFROMOID_DO.commit( dbt );
	    } else {
		// since the referenced DO is not loaded,
		// it cannot be dirty, so there is no need to commit it.
	    }
	} else {
	    if ( ! false )
		throw new RefAssertionException(
		    "Cannot commit StateTransitionDO ( " + toString() +
		    " ) because ENUMVALUEDEFINITIONFROMOID is not allowed to be null." );
	}
	org.webdocwf.util.genericstore.data.EnumValueDefinitionDO ENUMVALUEDEFINITIONTOOID_DO = getENUMVALUEDEFINITIONTOOID();
	if ( null != ENUMVALUEDEFINITIONTOOID_DO ) {
	    if ( ENUMVALUEDEFINITIONTOOID_DO.isLoaded() ) {
		okToCommitENUMVALUEDEFINITIONTOOID( ENUMVALUEDEFINITIONTOOID_DO );
		ENUMVALUEDEFINITIONTOOID_DO.commit( dbt );
	    } else {
		// since the referenced DO is not loaded,
		// it cannot be dirty, so there is no need to commit it.
	    }
	} else {
	    if ( ! false )
		throw new RefAssertionException(
		    "Cannot commit StateTransitionDO ( " + toString() +
		    " ) because ENUMVALUEDEFINITIONTOOID is not allowed to be null." );
	}

// WebDocWf extension for save without references
// The following line has been inserted
	  }
// end of WebDocWf extension for save without references
      }
      if ( false ) {
	  // This throw is here to keep the compiler happy
	  // in the case of a DO that does not refer to other DOs.
	  // In that case, the above delete/commit code blocks will be empty
	  // and throw nothing.
	  throw new DataObjectException( "foo" );	  
      }
      if ( delete ) {
          dbt.delete( this );
      } else {
	  if ( isLoaded() )
	      dbt.insert( this );   // dbt.insert() handles insertions and updates
      }
      if (ownTransaction) {
	  dbt.commit(); // commit the transaction
      }
    } catch (SQLException sqle) {
      StringBuffer message = new StringBuffer("Failed to insert/update DO: ");
      message.append(sqle.getMessage());

      // rollback, if necessary
      if (ownTransaction) {
        try {
          dbt.rollback();
        } catch (SQLException sqle2) {
          message.insert(0,"\n");
          message.insert(0,sqle2.getMessage());
          message.insert(0,"Rollback failed: ");
        }
      }
      throw new SQLException(message.toString());
    } finally {
      // release the transaction, if any
      if (ownTransaction) {
        dbt.release();
      }
    }
  }
}
