?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:common="org.enhydra.dods.Common"
                version="1.0">
	<xsl:output method="text" encoding="iso-8859-1"/>
	<xsl:template name="getRefererrersDO">
    /**
     * Get array of <xsl:value-of select="../@name"/>DO objects that refer to this DO.
     *
     * @return Array of <xsl:value-of select="../@name"/>DO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/> ()
    throws DataObjectException, QueryException {
	<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] ret = null;
		try {
		    <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
   			if (originDatabase != null)
			    q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else
</xsl:if>
			    q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
		    q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
		    ret = q.getDOArray();
		} catch ( NonUniqueQueryException e ) {
		    throw new DataObjectException( "INTERNAL ERROR: unexpected NonUniqueQueryException" );
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch (DatabaseManagerException e) {
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
</xsl:if>
		} finally {
		    if ( null == ret )
			ret = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[ 0 ];
		}
		return ret;
    }

    /**
     * Get single <xsl:value-of select="../@name"/>DO object that refers to this DO.
     *
     * @return <xsl:value-of select="../@name"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one <xsl:value-of select="../@name"/>DO object was found.
     */
    public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO <xsl:value-of select="dodst:getSingleDO(../@name,@name)"/> ()
    throws DataObjectException, NonUniqueQueryException, QueryException {
	<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch (DatabaseManagerException e) {
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		}
</xsl:if>
		q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
		q.requireUniqueInstance();
		return q.getNextDO();
    }

	/**
	 * Get array of <xsl:value-of select="../@name"/>DO objects that refer to this DO.
	 *
	 * @return Array of <xsl:value-of select="../@name"/>DO objects.
	 *
	 * @exception DataObjectException
	 *   If the object is not found in the database.
	 * @exception QueryException
	 *   If an error occured while building the query before execution.
	 */
	public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/> ()
	throws DataObjectException, QueryException {
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] ret = null;
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
			q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
			ret = q.getDOArray();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch ( DatabaseManagerException e ) {
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		} finally {
</xsl:if>
			if ( null == ret )
			ret = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[ 0 ];
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		}
</xsl:if>
		return ret;
	}

	/**
	 * Get single <xsl:value-of select="../@name"/>DO object that refers to this DO.
	 *
	 * @return <xsl:value-of select="../@name"/>DO object.
	 *
	 * @exception DataObjectException
	 *   If the object is not found in the database.
	 * @exception QueryException
	 *   If an error occured while building the query before execution.
	 * @exception NonUniqueQueryException
	 *   If more than one <xsl:value-of select="../@name"/>DO object was found.
	 */
	public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO <xsl:value-of select="dodst:getSingleDO(../@name,@name)"/> ()
	throws DataObjectException, NonUniqueQueryException, QueryException {
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch ( DatabaseManagerException e ) {
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		}
</xsl:if>
		q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
		q.requireUniqueInstance();
		return q.getNextDO();
	}

	</xsl:template>
</xsl:stylesheet>

